#include "mali_cv/Pyramid.h"

#include "mali_cv/Error.h"
#include "mali_cv/Helper.h"
#include "mali_cv/Image.h"

using namespace mali_cv;

CLPyramid::CLPyramid() : pyramid()
{
}

void CLPyramid::internal_create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format fmt )
{
	arm_compute::PyramidInfo pyramid_info;
	pyramid_info.init(levels, scale, w, h, ARM_COMPUTEHelper::convert_to_arm_compute_format(fmt));
	pyramid.init_auto_padding(pyramid_info);
	pyramid.allocate();
}

void CLPyramid::store( std::vector<ImagePtr> &out )
{
	ERROR_ON(out.size() != pyramid.info()->num_levels() );
	auto q = cl::CommandQueue::getDefault();

	for(uint i = 0; i < pyramid.info()->num_levels(); i++)
	{
		ERROR_ON( out[i]->cl.image.info()->dimension(0) != pyramid.get_pyramid_level(i)->info()->dimension(0) );
		ERROR_ON( out[i]->cl.image.info()->dimension(1) != pyramid.get_pyramid_level(i)->info()->dimension(1) );
		pyramid.get_pyramid_level(i)->map(q, true);
		out[i]->cl.image.map(q, true);
		out[i]->cl.image.copy_from(*pyramid.get_pyramid_level(i));
		out[i]->cl.image.unmap(q);
		pyramid.get_pyramid_level(i)->unmap(q);
	}
}
