#include <mali_cv/Error.h>
#include <mali_cv/RemapTable.h>
#include <mali_cv/Image.h>

using namespace mali_cv;

CLRemapTable::CLRemapTable():table()
{
}

void CLRemapTable::internal_create(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height)
{
	table.clear();
	table.resize(2);

	/* Allocate memory for mapx */
	table[0] = CLImageAlloc();
	table[0]->create(ctx, dst_width, dst_height, Format::FP32C1);

	/* Allocate memory for mapy */
	table[1] = CLImageAlloc();
	table[1]->create(ctx, dst_width, dst_height, Format::FP32C1);
}

void CLRemapTable::set_remap_point(int dst_x, int dst_y, float src_x, float src_y)
{
	/* Get pointers*/
	const auto mapx_ptr = reinterpret_cast<float*>(table[0]->image.ptr_to_element(arm_compute::Coordinates(dst_x, dst_y)));
	const auto mapy_ptr = reinterpret_cast<float*>(table[1]->image.ptr_to_element(arm_compute::Coordinates(dst_x, dst_y)));

	/* Set X coordinate */
	*mapx_ptr = src_x;

	/* Set Y coordinate */
	*mapy_ptr = src_y;
}

void CLRemapTable::map()
{
	auto q = cl::CommandQueue::getDefault();

	/* Map tables */
	table[0]->image.map(q);
	table[1]->image.map(q);
}

void CLRemapTable::unmap()
{
	auto q = cl::CommandQueue::getDefault();

	/* Unmap tables */
	table[0]->image.unmap(q);
	table[1]->image.unmap(q);
}
