#include "mali_cv/Tensor.h"
#include "mali_cv/Error.h"
#include "mali_cv/Context.h"
#include "mali_cv/Helper.h"
#include "arm_compute/core/Utils.h"
#include <cstring>

using namespace mali_cv;

inline void fill(unsigned char *in_buffer, arm_compute::ICLTensor &out)
{
	ERROR_ON(nullptr == in_buffer);

	uint w = out.info()->dimension(0);
	uint h = out.info()->dimension(1);
	int element_size = out.info()->element_size();
	int collapsed_upper = out.info()->tensor_shape().total_size_upper(2);

	for(int i = 0; i < collapsed_upper; ++i)
	{
		for(unsigned int y = 0; y < h; ++y)
		{
			memcpy(out.buffer() + out.info()->offset_first_element_in_bytes() + i *  out.info()->strides_in_bytes().z() + y * out.info()->strides_in_bytes().y(),
			       in_buffer + i * w * h * element_size + y * w * element_size,
			       w * element_size );
		}
	}
}

inline void collect(unsigned char *out_buffer, arm_compute::ICLTensor &in)
{
	ERROR_ON(nullptr == out_buffer);

	uint w = in.info()->dimension(0);
	uint h = in.info()->dimension(1);
	int element_size = in.info()->element_size();
	int collapsed_upper = in.info()->tensor_shape().total_size_upper(2);

	for(int i = 0; i < collapsed_upper; ++i)
	{
		for(unsigned int y = 0; y < h; ++y)
		{
			memcpy(out_buffer + i * w * h * element_size + y * w * element_size,
			       in.buffer() + in.info()->offset_first_element_in_bytes() + i *  in.info()->strides_in_bytes().z() + y * in.info()->strides_in_bytes().y(),
			       w * element_size );
		}
	}
}

CLTensor::CLTensor(): ITensor(), tensor()
{
}

void CLTensor::internal_create(Context &ctx, TensorShape shape, DataType data_type, size_t num_channels)
{
	arm_compute::TensorInfo info(shape, num_channels, data_type);
	info.auto_padding();
	tensor.allocator()->init(info);
	tensor.allocator()->allocate();
}

void CLTensor::load(uint8_t *in)
{
	ERROR_ON(nullptr == in);
	auto q = cl::CommandQueue::getDefault();
	tensor.map(q, true);
	fill(in, tensor);
	tensor.unmap(q);
}

void CLTensor::store(uint8_t *out)
{
	ERROR_ON(nullptr == out);
	auto q = cl::CommandQueue::getDefault();
	tensor.map(q, true);
	collect(out, tensor);
	tensor.unmap(q);
}

int CLTensor::size()
{
	return _shape.total_size() * _num_channels;
}
