#include "mali_cv/Convolution.h"

#include "mali_cv/Error.h"
#include "mali_cv/RawImage.h"
#include "mali_cv/Helper.h"

using namespace mali_cv;

CVConvolution::CVConvolution():IConvolution(),type(0),convolution(),_scale(0)
{
}

void CVConvolution::internal_create( Context &ctx, size_t w, size_t h, Format format)
{
	type = CVHelper::convert_to_cv_format( format );
	convolution = cv::Mat(h, w, type);
}

void CVConvolution::store( RawImage &out )
{
	Format format = Format::UNKNOWN;
	switch( type )
	{
		case CV_8U:
			format = Format::LUMINANCE;
			break;
		case CV_16S:
			format = Format::LUMINANCE16;
			break;
		case CV_8UC3:
			format = Format::RGB888;
			break;
		case CV_8UC4:
			format = Format::RGBA8888;
			break;
		default:
			ERROR("Not supported");
	}

	out.create( width, height, format);

	memcpy( out.data(), convolution.data, out.num_pixels()*out.pixel_size());
}

void CVConvolution::load(const RawImage &in, uint32_t scale )
{
	ERROR_ON( static_cast<size_t>(convolution.rows) != in.height() );
	ERROR_ON( static_cast<size_t>(convolution.cols) != in.width() );

	switch(type)
	{
		case CV_8U:
			if(in.pixel_size() == 1)
				memcpy( convolution.data, in.data(), in.num_pixels());
			break;
		case CV_16S:
			if(in.pixel_size() == 2)
				memcpy( convolution.data, in.data(), in.num_pixels() * in.pixel_size());
			break;
		case CV_8UC2:
			/* TODO */
			break;
		case CV_8UC3:
			/* TODO */
			break;
		case CV_8UC4:
			/* TODO */
			break;
		default:
		{
			ERROR_ON( in.pixel_size() != 1);

			const auto out = reinterpret_cast<int16_t*>(convolution.data);

			for(uint i = 0; i < in.num_pixels(); i++)
			{
				out[i] = in.data()[i];
			}
		}
	}
	_scale = scale;
}
