#include "mali_cv/Convolution.h"

#include "mali_cv/RawImage.h"
#include "mali_cv/Context.h"

using namespace mali_cv;

IConvolution::IConvolution():width(0),height(0),format(Format::UNKNOWN)
{
}

void IConvolution::create( Context &ctx, const RawImage &in, CopyData copy)
{
	create(ctx, in.width(), in.height(), in.format());

	if( copy == CopyData::YES)
	{
		load( in,0 );
	}
}

void IConvolution::create( Context &ctx, size_t w, size_t h, Format fmt)
{
	width = w;
	height = h;
	format = fmt;
	internal_create(ctx, w, h, fmt);
}

Convolution::Convolution( Context &ctx, const RawImage &in, CopyData copy):cv(),vx(),cl(),ne()
{
	cl.create(ctx, in, copy);
	cv.create(ctx, in, copy);
	vx.create(ctx, in, copy);
	ne.create(ctx, in, copy);
}

Convolution::Convolution( Context &ctx, size_t width, size_t height, Format format):cv(),vx(),cl(),ne()
{
	cv.create(ctx, width, height, format);
	vx.create(ctx, width, height, format);
	cl.create(ctx, width, height, format);
	ne.create(ctx, width, height, format);
}

void Convolution::load(const RawImage &in, uint32_t scale)
{
	cv.load( in, scale );
	vx.load( in, scale );
	cl.load( in, scale );
	ne.load( in, scale );
}
