#include "mali_cv/Tensor.h"
#include "mali_cv/Error.h"
#include "mali_cv/Context.h"
#include "mali_cv/Helper.h"
#include "arm_compute/core/Utils.h"

using namespace mali_cv;

#ifdef USE_EIGEN

EigenTensor::EigenTensor(): ITensor(), tensor()
{
}

void EigenTensor::internal_create(Context &ctx, TensorShape shape, DataType data_type, size_t num_channels)
{
	tensor = Eigen::Tensor<float, MAX_DIMS, Eigen::RowMajor>(
		static_cast<int>(shape[5]),
		static_cast<int>(shape[4]),
		static_cast<int>(shape[3]),
		static_cast<int>(shape[2]),
		static_cast<int>(shape[1]),
		static_cast<int>(shape[0] * num_channels));
}

void EigenTensor::load(uint8_t *in)
{
	ERROR_ON(nullptr == in);
	memcpy(tensor.data(), in, tensor.size() * sizeof(float));
}

void EigenTensor::store(uint8_t *out)
{
	ERROR_ON(nullptr == out);
	memcpy(out, tensor.data(), tensor.size() * sizeof(float));
}

int EigenTensor::size()
{
    return tensor.size();
}

#endif
