#include "mali_cv/File.h"

#include "mali_cv/Error.h"

#include <cstdarg>
#include <cstdlib>
#include <cerrno>
#include <cstring>

using namespace mali_cv;

File::File(const std::string &filename, const std::string &mode): _fd(fopen(filename.c_str(), mode.c_str()), &fclose), _size(0), _filename(filename)
{
	ERROR_ON_MSG(! _fd, "Failed to open '%s' (%s [%d])", _filename.c_str(), strerror(errno), errno);
}

void File::open(const std::string &filename, const std::string &mode)
{
	close();

	_filename = filename;
	_fd = std::unique_ptr<FILE, decltype(&fclose)>(fopen(_filename.c_str(), mode.c_str()), &fclose);
	ERROR_ON_MSG(! _fd, "Failed to open '%s' (%s [%d])", _filename.c_str(), strerror(errno), errno);
}

void File::close()
{
	_filename.clear();
	_fd = nullptr;
	_size = 0;
}

size_t File::size() const
{
	if(_size == 0 && _fd)
	{
		long pos = ftell(_fd.get());

		if(fseek(_fd.get(), 0, SEEK_END))
		{
			ERROR("Failed to read the size of %s", _filename.c_str());
		}

		_size = ftell(_fd.get());
		
		if(fseek(_fd.get(), pos, SEEK_SET))
		{
			ERROR("Failed to read the size of %s", _filename.c_str());
		}
	}

	return _size;
}

void File::read( void *data, size_t size )
{
	if( fread(data, size, 1,  _fd.get()) != 1 )
	{
		ERROR("Failed to load %zu bytes from %s (%s [%d])", size, _filename.c_str(), strerror(errno), errno);
	}
}

void File::write( const void *data, size_t size )
{
	if( fwrite(data, size, 1, _fd.get()) != 1 )
	{
		ERROR("Failed to write %zu bytes to %s (%s [%d])", size, _filename.c_str(), strerror(errno), errno);
	}
}
FILE *File::fd()
{
	return _fd.get();
}
void File::printf( const char *msg, ...)
{
	va_list args;
	va_start(args, msg);
	if( vfprintf( _fd.get(), msg, args) < 0 )
		ERROR("Failed to write to %s", _filename.c_str());
	va_end(args);
}

void File::scanf( const char *format, ...)
{
	va_list args;
	va_start(args, format);
	if( EOF == vfscanf( _fd.get(), format, args))
		ERROR("Failed to read from %s", _filename.c_str());
	va_end(args);
}

char File::getc()
{
	return ::getc(_fd.get());
}

void File::ungetc( char c )
{
	::ungetc( c, _fd.get() );
}
