#include "mali_cv/Helper.h"

#include <cstdlib>
#include <random>
#include <sstream>
#include <string>
#include <vector>

namespace mali_cv
{
std::vector<int> split_string_to_int(std::string input, char delimiter)
{
	std::vector<int> output;
	std::istringstream ss(input);
	std::string token;

	while(std::getline(ss, token, delimiter))
	{
		output.push_back(std::atoi(token.c_str()));
	}

	return output;
}

TensorShape vector_to_tensorshape(std::vector<int> v)
{
	TensorShape shape;

	for(unsigned int i = 0, e = std::min(v.size(), arm_compute::MAX_DIMS); i < e; ++i)
	{
		shape.set(i, v.at(i));
	}

	return shape;
}

unsigned int random_seed()
{
	static unsigned int seed = 0;

	if(0 == seed)
	{
		std::random_device rd;
		READ_CONFIG(seed, "RANDOM_SEED", rd());
	}
	return seed++;
}

std::string format_to_string(const Format &fmt)
{
    switch(fmt)
    {
        case Format::U8:
            return "U8";
        case Format::S8:
            return "S8";
        case Format::U16:
            return "U16";
        case Format::S16:
            return "S16";
        case Format::U32:
            return "U32";
        case Format::S32:
            return "S32";
        case Format::FP32C1:
            return "FP32C1";
        case Format::FP32C2:
            return "FP32C2";
        case Format::FP16C1:
            return "FP16C1";
        case Format::FP16C2:
            return "FP16C2";
        case Format::IYUV:
            return "IYUV";
        case Format::UYVY:
            return "UYVY";
        case Format::YUV444:
            return "YUV444";
        case Format::YUYV422:
            return "YUYV422";
        case Format::NV12:
            return "NV12";
        case Format::NV21:
            return "NV21";
        case Format::RGB888:
            return "RGB888";
        case Format::RGBA16:
            return "RGBA16";
        case Format::RGBA8888:
            return "RGBA8888";
        case Format::RGBA_FLOAT:
            return "RGBA_FLOAT";
        default:
            ARM_COMPUTE_ERROR("Unsupported input format");
            return "";
    }
}
}
