#include "mali_cv/Image.h"

#include "mali_cv/Error.h"
#include "mali_cv/RawImage.h"
#include "mali_cv/Context.h"

using namespace mali_cv;

IImage::IImage():width(0),height(0),format(Format::UNKNOWN)
{
}

void IImage::create( Context &ctx, const RawImage &in, CopyData copy)
{
	create(ctx, in.width(), in.height(), in.format());
	if( copy == CopyData::YES)
	{
		load( in );
	}
}

void IImage::create( Context &ctx, size_t w, size_t h, Format fmt)
{
	width = w;
	height = h;
	format = fmt;
	internal_create(ctx, w, h, fmt);
}

Image::Image( Context &ctx, const RawImage &in, CopyData copy):cv(),vx(),cl(),ne(), width{in.width()}, height{in.height()}
{
	cl.create(ctx, in, copy);
	cv.create(ctx, in, copy);
	vx.create(ctx, in, copy);
	ne.create(ctx, in, copy);
}

Image::Image( Context &ctx, const RawImage *in, CopyData copy):cv(),vx(),cl(),ne()
{
	if(in)
	{
		cl.create(ctx,*in, copy);
		cv.create(ctx,*in, copy);
		vx.create(ctx,*in, copy);
		ne.create(ctx,*in, copy);

        width = in->width();
        height = in->height();
	}
}

Image::Image( Context &ctx, size_t width, size_t height, Format format):cv(),vx(),cl(),ne(), width{width}, height{height}
{
	cv.create(ctx, width, height, format);
	vx.create(ctx, width, height, format);
	cl.create(ctx, width, height, format);
	ne.create(ctx, width, height, format);
}

void Image::load(const RawImage &in)
{
	cv.load(in);
	vx.load(in);
	cl.load(in);
	ne.load(in);
}

#if defined(USE_NEON) || defined(USE_OPENCL)
unsigned char* IImage::copy_plane(arm_compute::IImage& in, unsigned char* out_buffer)
{
	ERROR_ON(nullptr == out_buffer);

	for(unsigned int y = 0; y < in.info()->dimension(1); ++y)
	{
		// Use width and pixel_size from input rather than output.
		// It is because we are addressing based on per plane not per whole image
		memcpy(out_buffer + in.info()->dimension(0) * in.info()->element_size() * y,
		       in.buffer() + in.info()->offset_element_in_bytes(arm_compute::Coordinates(0, static_cast<int>(y))),
		       in.info()->dimension(0) * in.info()->element_size());
	}

	return out_buffer + in.info()->dimension(0) * in.info()->element_size() * in.info()->dimension(1);
}

unsigned char* IImage::fill_plane(unsigned char* in_buffer, arm_compute::IImage& out)
{
	ERROR_ON(nullptr == in_buffer);

	for(unsigned int y = 0; y < out.info()->dimension(1); ++y)
	{
		// Use width and pixel_size from output rather than input.
		// It is because we are addressing based on per plane not per whole image
		memcpy(out.buffer() + out.info()->offset_element_in_bytes(arm_compute::Coordinates(0, static_cast<int>(y))),
		       in_buffer + out.info()->dimension(0) * out.info()->element_size() * y,
		       out.info()->dimension(0) * out.info()->element_size());
	}

	return in_buffer + out.info()->dimension(0) * out.info()->element_size() * out.info()->dimension(1);
}
#endif /* defined(USE_NEON) || defined(USE_OPENCL) */
