#ifndef _NECOMPUTE_HELPER_HEADER_
#define _NECOMPUTE_HELPER_HEADER_

#include <mali_cv/RawImage.h>
#include <mali_cv/Image.h>
#include <mali_cv/Context.h>

#define NE_DEBUG_ENABLED 0


#if NE_DEBUG_ENABLED
#include <iostream>
#include <iomanip>

#define _NE_CHECK_TEXEL_VALUE( ref_value, neon_result, x, y, ctx)\
	if( neon_result != ref_result ) {\
		using namespace _ne_debug;\
		std::cout << "x " << x << " y " << y << std::endl;\
		std::cout << "neon " << neon_result << " correct " << ref_result << std::endl;\
		_TexelCoord neighs[9];\
		unsigned char values[9];\
		_get_3x3_coords( _TexelCoord(x,y), neighs );\
		_get_values(ctx, *(args.a),  neighs,9, values);\
		std::cout << "values : " << std::endl;\
		std::cout << (int) values[0] << " " <<  (int) values[1] << " " << (int) values[2] << std::endl;\
		std::cout << (int) values[3] << " " <<  (int) values[4] << " " << (int) values[5] << std::endl;\
		std::cout << (int) values[6] << " " <<  (int) values[7] << " " << (int) values[8] << std::endl;\
		ERROR_ON( neon_result != ref_result );\
	}

#define _NE_DUMP_VECTOR(v,t) _ne_debug::dump(v,t)

namespace _ne_debug {
	inline bool are_equal( uint8x16_t d, const uint8_t* buf, size_t n=16) {
		uint8_t tmp[16];
		vst1q_u8(tmp, d);
		return 0 == memcmp(tmp,buf,n);
	}
	inline void dump( uint64x2_t d, const char *s) {
			uint64_t tmp[2];
			vst1q_u64(tmp, d);
			std::cout << std::endl << s << " (uint64x2_t) : " ;
			for (auto t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}

	inline void dump( uint8x16_t d, const char *s) {
			uint8_t tmp[16];
			vst1q_u8(tmp, d);
			std::cout << std::endl << s << " (uint8x16_t) : " ;
			for (auto t : tmp ) std::cout << (int) t << " ";
			std::cout << std::endl;
	}
	inline void dump( uint16x8_t d, const char *s) {
			uint16_t tmp[8];
			vst1q_u16(tmp, d);
			std::cout << std::endl << s << " (uint16x8_t) : " ;
			for (auto t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}
	inline void dump( float32x4_t d, const char *s) {
			float tmp[4];
			vst1q_f32(tmp, d);
			std::cout << std::endl << s << " (float32x4_t) : " ;
			for (float t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}

	inline void dump( int16x4_t d, const char *s) {
			int16_t tmp[4];
			vst1_s16(tmp, d);
			std::cout << std::endl << s << " (int16x4_t) : " ;
			for (auto t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}
	inline void dump( int16x8_t d, const char *s) {
			int16_t tmp[8];
			vst1q_s16(tmp, d);
			std::cout << std::endl << s << " (int16x8_t) : " ;
			for (auto t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}

	inline void dump( int32x2_t d, const char *s) {
			int32_t tmp[2];
			vst1_s32(tmp, d);
			std::cout << std::endl << s << " (uint32x2_t) : " ;
			for (auto t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}
	inline void dump( int32x4_t d, const char *s) {
			int32_t tmp[4];
			vst1q_s32(tmp, d);
			std::cout << std::endl << s << " (uint32x4_t) : " ;
			for (auto t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}

	inline void dump( uint16x4_t d, const char *s) {
			uint16_t tmp[4];
			vst1_u16(tmp, d);
			std::cout << std::endl << s << " (uint16x4_t) : " ;
			for (auto t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}
	inline void dump( uint32x2_t d, const char *s) {
			uint32_t tmp[2];
			vst1_u32(tmp, d);
			std::cout << std::endl << s << " (uint32x2_t) : " ;
			for (auto t : tmp ) std::cout << t << " ";
			std::cout << std::endl;
	}

	inline void dump( uint8x8_t d, const char *s) {
			uint8_t _tmp[8];
			vst1_u8(_tmp, d);
			std::cout << std::endl << s << " (uint8x8_t) : " ;
			for (auto t : _tmp ) std::cout << (int) t << " ";
			std::cout << std::endl;
	}
	template<typename T>  
	inline void dump_img( T* pBuffer, size_t width, size_t height) {
		std::cout << std::endl << std::endl;
		for(size_t k(0); k < height ; ++k ) {

			for( size_t j(0); j < width ; ++j ) {
				std::cout <<  std::setw(5) << std::setfill('0') <<  (int) pBuffer[ j + k * width] << " ";
			}
			std::cout << std::endl;

		}
	}
	inline void dump_image( arm_compute::IImage& i, bool show_border = true ) {
		switch(i.image_info()->format() )
		{
			case arm_compute::Format::U8:
			{
				const uint8_t* buffer;
				if( show_border) buffer = i.buffer();
				else buffer =  i.buffer() +  i.image_info()->offset_first_element_in_bytes() ;
				for(size_t j=0; j < i.image_info()->height(); ++j ) {
					for(size_t k=0; k < i.image_info()->width(); ++k) {
						//std::cout <<  std::setw(8) << std::setfill('0') << *((const float*)&buffer[j*i.img.image_info()->stride_in_bytes()+k*4]) << " " ;
						printf("%03d ", *((const uint8_t*)&buffer[j*i.image_info()->stride_in_bytes()+k]) ) ;

					}
					std::cout << std::endl;
				}
				break;


			}
			case arm_compute::Format::S32:
			case arm_compute::Format::F32:
			{
				const uint8_t* buffer;
				if( show_border) buffer = i.buffer();
				else buffer =  i.buffer() +  i.image_info()->offset_first_element_in_bytes() ;
				for(size_t j=0; j < i.image_info()->height(); ++j ) {
					for(size_t k=0; k < i.image_info()->width(); ++k) {
						//std::cout <<  std::setw(8) << std::setfill('0') << *((const float*)&buffer[j*i.img.image_info()->stride_in_bytes()+k*4]) << " " ;
						printf("%f ", *((const float*)&buffer[j*i.image_info()->stride_in_bytes()+k*4]) ) ;

					}
					std::cout << std::endl;
				}
				break;
			}
			default:
				break;
		}

	}
	inline void dump_image( mali_cv::NEImage& i ) {
		using namespace mali_cv;
		std::cout << " image " << (&i) << std::endl;
		dump_image(i.image);
		std::cout << std::endl;

	}



}

#else
#define _NE_CHECK_TEXEL_VALUE( ref_value, neon_result, x, y) {}
#define _NE_DUMP_VECTOR(v,t) {}

#endif


namespace _ne_debug {
	using namespace mali_cv;

	typedef struct _TexelCoord {
		size_t x;
		size_t y;
		_TexelCoord(size_t tx, size_t ty) : x(tx), y(ty) {}
		_TexelCoord() : x(0), y(0) {}
	} _TexelCoord;

	inline bool _is_valid_coord(const _TexelCoord &texel , size_t width, size_t height ) 
	{ 
		return texel.x < width && texel.y < height ; 
	}

	inline bool _is_valid_coord(const _TexelCoord &texel , const mali_cv::NEImage &img ) 
	{ 
		return  _is_valid_coord(texel , img.width , img.height );
	}


	template<typename T>
	T _clamp(const T &val, const T &min, const T &max)
	{
		return std::max(min, std::min(max, val));
	}


	inline void _get_3x3_coords(const _TexelCoord &t, _TexelCoord *neighs)
	{
			neighs[0] = _TexelCoord(t.x-1, t.y-1);
			neighs[1] = _TexelCoord(t.x  , t.y-1);
			neighs[2] = _TexelCoord(t.x+1, t.y-1);
			neighs[3] = _TexelCoord(t.x-1 , t.y);
			neighs[4] = _TexelCoord(t.x,  t.y);
			neighs[5] = _TexelCoord(t.x+1,  t.y);
			neighs[6] = _TexelCoord(t.x-1, t.y+1);
			neighs[7] = _TexelCoord(t.x  , t.y+1);
			neighs[8] = _TexelCoord(t.x+1, t.y+1);
	}

	inline void _get_5x5_coords(const _TexelCoord &t, _TexelCoord *neighs)
	{
			neighs[0] = _TexelCoord(t.x-2, t.y-2);
			neighs[1] = _TexelCoord(t.x-1, t.y-2);
			neighs[2] = _TexelCoord(t.x  , t.y-2);
			neighs[3] = _TexelCoord(t.x+1, t.y-2);
			neighs[4] = _TexelCoord(t.x+2, t.y-2);

			neighs[5] = _TexelCoord(t.x-2, t.y-1);
			neighs[6] = _TexelCoord(t.x-1, t.y-1);
			neighs[7] = _TexelCoord(t.x  , t.y-1);
			neighs[8] = _TexelCoord(t.x+1, t.y-1);
			neighs[9] = _TexelCoord(t.x+2, t.y-1);

			neighs[10] = _TexelCoord(t.x-2 , t.y);
			neighs[11] = _TexelCoord(t.x-1 , t.y);
			neighs[12] = _TexelCoord(t.x  ,  t.y);
			neighs[13] = _TexelCoord(t.x+1,  t.y);
			neighs[14] = _TexelCoord(t.x+2,  t.y);

			neighs[15] = _TexelCoord(t.x-2, t.y+1);
			neighs[16] = _TexelCoord(t.x-1, t.y+1);
			neighs[17] = _TexelCoord(t.x  , t.y+1);
			neighs[18] = _TexelCoord(t.x+1, t.y+1);
			neighs[19] = _TexelCoord(t.x+2, t.y+1);

			neighs[20] = _TexelCoord(t.x-2, t.y+2);
			neighs[21] = _TexelCoord(t.x-1, t.y+2);
			neighs[22] = _TexelCoord(t.x  , t.y+2);
			neighs[23] = _TexelCoord(t.x+1, t.y+2);
			neighs[24] = _TexelCoord(t.x+2, t.y+2);

	}
	inline size_t _texcoord_to_index(size_t stride, size_t pixel_size, const _TexelCoord &texel)  
	{ 
		return texel.y*stride + texel.x*pixel_size ; 
	}

	template<typename T>
	inline void _get_values(const Context &ctx, const uint8_t* buffer, size_t stride, const _TexelCoord *neighs, size_t neighs_size, T*values)
	{
		ERROR_ON(nullptr ==  buffer);
		ERROR_ON(nullptr ==  values);
		for( size_t j(0); j < neighs_size ; ++j ) {		
			const size_t n_index =  _texcoord_to_index( stride, sizeof(T), neighs[j] );
			values[j] = *( (T*) (&buffer[n_index]) ) ;
		} 
	}


}



#endif
