#include "mali_cv/Convolution.h"

#include "support/ToolchainSupport.h"
#include "mali_cv/Error.h"
#include "mali_cv/RawImage.h"

#include <cstring>

using namespace mali_cv;

NEConvolution::NEConvolution():IConvolution(),convolution(),_scale(0)
{
}

void NEConvolution::internal_create(Context &ctx, size_t w, size_t h, Format format)
{
	ERROR_ON(format != Format::LUMINANCE16);
	ERROR_ON(convolution != nullptr);

	convolution = arm_compute::support::cpp14::make_unique<int16_t[]>(w * h);
}

void NEConvolution::store( RawImage &out )
{
	out.create( width, height, Format::LUMINANCE16);
	memcpy( out.data(), convolution.get(), out.num_pixels() * out.pixel_size());
}

void NEConvolution::load( const RawImage &in, uint32_t scale  )
{
	ERROR_ON(in.format() != Format::LUMINANCE16);

	memcpy(convolution.get(), in.data(), in.num_pixels() * in.pixel_size());
	_scale = scale;
}
