#include "mali_cv/Pyramid.h"

#include "mali_cv/Error.h"
#include "mali_cv/Context.h"
#include "mali_cv/Image.h"

using namespace mali_cv;

void IPyramid::create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format fmt )
{
	internal_create(ctx, levels, scale, w, h, fmt);
}

Pyramid::Pyramid( Context &ctx, size_t levels, float scale_factor, size_t w, size_t h, Format fmt): cv(), vx(), cl(), ne(), n_levels(levels), width(w), height(h), format(fmt), scale(scale_factor), context(ctx)
{
	ERROR_ON( Format::IYUV    == format );
	ERROR_ON( Format::NV12    == format );
	ERROR_ON( Format::NV21    == format );
	ERROR_ON( Format::YUV444  == format );
	ERROR_ON( Format::YUYV422 == format );

	cv.create(ctx, n_levels, scale, width, height, format);
	vx.create(ctx, n_levels, scale, width, height, format);
	cl.create(ctx, n_levels, scale, width, height, format);
	ne.create(ctx, n_levels, scale, width, height, format);
}

void Pyramid::store( std::vector<ImagePtr> &out)
{
	out.clear();
	size_t w = width;
	size_t h = height;

	for( size_t i = 0; i < n_levels; i++)
	{
		ERROR_ON( w < 16 || h < 16);
		out.push_back( std::make_shared<Image>(context, w, h, format));
		w = (w + 1) * scale;
		h = (h + 1) * scale;
	}

	cv.store( out );
	vx.store( out );
	cl.store( out );
	ne.store( out );
}
