#include "mali_cv/RemapTable.h"

#include "mali_cv/Context.h"
#include "mali_cv/Error.h"

using namespace mali_cv;

void IRemapTable::create(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height)
{
	internal_create(ctx, src_width, src_height, dst_width, dst_height);
}

RemapTable::RemapTable(Context &ctx, size_t src_w, size_t src_h, size_t dst_w, size_t dst_h): cv(),vx(),cl(),ne(), src_width(src_w),src_height(src_h),dst_width(dst_w),dst_height(dst_h),context(ctx)
{
	cv.create(ctx, src_width, src_height, dst_width, dst_height);
	vx.create(ctx, src_width, src_height, dst_width, dst_height);
	cl.create(ctx, src_width, src_height, dst_width, dst_height);
	ne.create(ctx, src_width, src_height, dst_width, dst_height);
}

void RemapTable::set_remap_point(int dst_x, int dst_y, float src_x, float src_y)
{
	ERROR_ON( (dst_x >= static_cast<int>(dst_width)) || (dst_y >= static_cast<int>(dst_height)) );
	ERROR_ON( (src_x < 0) || (src_y < 0) || (src_x >= src_width) || (src_y >= src_height) );

	cv.set_remap_point(dst_x, dst_y, src_x, src_y);
	vx.set_remap_point(dst_x, dst_y, src_x, src_y);
	cl.set_remap_point(dst_x, dst_y, src_x, src_y);
	ne.set_remap_point(dst_x, dst_y, src_x, src_y);
}

void RemapTable::map()
{
	cv.map();
	vx.map();
	cl.map();
	ne.map();
}

void RemapTable::unmap()
{
	cv.unmap();
	vx.unmap();
	cl.unmap();
	ne.unmap();
}
