#include "mali_cv/Tensor.h"

#include "mali_cv/Error.h"
#include "mali_cv/Context.h"
#include "mali_cv/Helper.h"

#include <random>

using namespace mali_cv;

inline std::vector<int> random_int(size_t size, int min, int max)
{
	std::vector<int> v(size);

	uint seed = random_seed();
	std::mt19937 rng( seed );
	std::uniform_int_distribution<int> gen( min, max );
	for(size_t i = 0; i < size; ++i)
	{
		v.at(i) = gen(rng);
	}
	return v;
}

inline std::vector<float> random_real(size_t size, int min, int max)
{
	std::vector<float> v(size);

	uint seed = random_seed();
	std::mt19937 rng( seed );
	std::uniform_real_distribution<float> gen( min, max );
	for(size_t i = 0; i < size; ++i)
	{
		v.at(i) = gen(rng);
	}
	return v;
}

ITensor::ITensor() : _shape(), _dt(DataType::UNKNOWN), _num_channels(1)
{
}

void ITensor::create(Context &ctx, TensorShape shape, DataType data_type, size_t num_channels)
{
	_dt = data_type;
	_shape = shape;
	_num_channels = num_channels;
	internal_create(ctx, _shape, _dt, _num_channels);
}

Tensor::Tensor() : _shape(), _dt(DataType::UNKNOWN), _num_channels(1), cv(), vx(), cl(), ne()
{
}

Tensor::Tensor(Context &ctx, TensorShape shape, DataType dt, size_t num_channels) : _shape(shape), _dt(dt), _num_channels(num_channels), cv(), vx(), cl(), ne()
{
	cl.create(ctx, _shape, _dt, _num_channels);
	cv.create(ctx, _shape, _dt, _num_channels);
	vx.create(ctx, _shape, _dt, _num_channels);
	ne.create(ctx, _shape, _dt, _num_channels);
}

void Tensor::init(Context& ctx, TensorShape shape, DataType dt, size_t num_channels)
{
	_shape = shape;
	_dt = dt;
	_num_channels = num_channels;
	cl.create(ctx, _shape, _dt, _num_channels);
	cv.create(ctx, _shape, _dt, _num_channels);
	vx.create(ctx, _shape, _dt, _num_channels);
	ne.create(ctx, _shape, _dt, _num_channels);
}

void Tensor::random_fill(float min, float max)
{
	std::vector<float> rand_v = random_real(_shape.total_size() * _num_channels, min, max);
	uint8_t *data = reinterpret_cast<uint8_t*>(rand_v.data());

	cl.load(data);
	cv.load(data);
	vx.load(data);
	ne.load(data);
}
