#ifndef _COMPUTER_VISION_RESULTS_HEADER_
#define _COMPUTER_VISION_RESULTS_HEADER_

#include <sys/types.h>
#include <unordered_map>
#include <vector>
#include <mali_cv/ITest.h>

namespace mali_cv
{
	class Environment;

// This class simply stores the time it took each computer vision to execute
class TestResults
{
	public:
	        static const std::string PMU_Cycles;

		static TestResults& report();

		TestResults();
		void init(uint num_iterations);
		void add(const std::string &target_name, float time );
		void add_pmu_counter(const std::string &name, long long value);
		void set_current_test( int test_id, const std::string& name);
		void save_to_file(const char* image, Environment &env) ;
	private:
		class PMUData
	        {
                       public:
                               PMUData() : _counters(),_median() {}
                               size_t add_counter( const std::string & name, long long value );
                               long long get_median( const std::string & name ) const;
                               void compute_stats();
                       private:
                               std::map<std::string, std::vector<long long> > _counters;
                               std::map<std::string, long long > _median;
               };


		void save_to_html( const std::string& filename, const char* image, Environment &env);
		void save_to_json( const std::string& filename, const char* image, Environment &env);
		class TimeData
		{
			public:
			TimeData(): _times(),_average(0.0f),_median(0.0f),_speedup(0.0f)
			{
			}
			uint add_time(float time);
			void compute_stats(float reference_time);

			const std::vector<float>& times()const;
			float average()const;
			float median()const;
			float speedup()const;
			private:
				std::vector<float> _times;
				float _average, _median, _speedup;
		};
		class TestData
		{
			public:
				TestData( int test_id, const std::string& name);
				const std::string& name()const
				{
					return _name;
				}
				std::string full_name()const
				{
					return _full_name;
				}
				TimeData &data( TargetID target);
				const TimeData &data( TargetID target)const;
				PMUData &pmu_counters();
                                const PMUData &pmu_counters() const;
				void compute_stats( TargetID reference);
			private:
				std::array<TimeData, (int)TargetID::NUM_TARGETS> _data;
				PMUData _pmu_counters;
				std::string _name, _full_name;
		};
		std::map<int /* Test ID */, TestData> _tests;
		std::array<std::string, (int)TargetID::NUM_TARGETS> _targets;
		uint _num_iterations;
		int  _current_test_id;
		std::string _current_test_name;
};
}
#endif
