#include "mali_cv/Context.h"

#include "mali_cv/Error.h"
#include "TestResults.h"

using namespace mali_cv;

namespace
{
void VX_CALLBACK log_callback( vx_context context, vx_reference ref, vx_status status, const vx_char string[] )
{
	printf( "LOG: [ status = %d ] %s", status, string );
	fflush( stdout );
}
}

VXContext::VXContext( ): IContext(),
_ctx(vxCreateContext())
{
	ERROR_ON(!_ctx);
	vxRegisterLogCallback( _ctx, log_callback, vx_false_e );
}

vx_border_t VXContext::vx_border_mode(BorderMode border_mode)const
{
	vx_border_t mode;
	mode.mode = VX_BORDER_UNDEFINED;
	switch(border_mode)
	{
		case BorderMode::UNDEFINED:
			printf("Border mode : UNDEFINED\n");
			mode.mode = VX_BORDER_UNDEFINED;
			break;
		case BorderMode::CONSTANT:
			mode.mode = VX_BORDER_CONSTANT;
			mode.constant_value.U8 = CONSTANT_BORDER_VALUE;
			printf("Border mode : CONSTANT (%d)\n", mode.constant_value.U8);
			break;
		case BorderMode::REPLICATE:
			printf("Border mode : REPLICATE\n");
			mode.mode = VX_BORDER_REPLICATE;
			break;
		default:
			ERROR("Border mode unknown");
	}
	return mode;
}

vx_graph VXContext::create_graph()
{
	vx_graph graph = vxCreateGraph(_ctx);
	ERROR_ON(!graph);
	return graph;
}

void VXContext::run_graph( vx_graph graph, uint num_iterations, const char* tag)
{
	CHECK_VX( vxVerifyGraph(graph) );
	// Don't time OpenVX: it is really slow and therefore meaningless
	// As performance is not measured: ignore num_iterations and process the graph only once.
	CHECK_VX( vxProcessGraph(graph));
	UNUSED(tag);
}

void VXContext::set_border_mode( vx_node node, BorderMode border_mode )
{
	ERROR_ON(!node);
	vx_border_t mode = vx_border_mode(border_mode);
	CHECK_VX( vxSetNodeAttribute( node, VX_NODE_BORDER, &mode, sizeof(mode)));
	CHECK_VX( vxReleaseNode( &node ) );
}

vx_context VXContext::context()
{
	return _ctx;
}

VXContext::~VXContext()
{
}
