#include <mali_cv/Context.h>
#include <mali_cv/RawImage.h>
#include <mali_cv/Error.h>
#include <mali_cv/Convolution.h>

using namespace mali_cv;

VXConvolution::VXConvolution():IConvolution(),convolution()
{
}

void VXConvolution::internal_create( Context &ctx, size_t w, size_t h, Format format)
{
	ERROR_ON(format != Format::LUMINANCE16);

	width = w;
	height = h;
	convolution = vxCreateConvolution(ctx.vx.context(), width, height);
	ERROR_ON(convolution == nullptr);
}

void VXConvolution::store( RawImage &out )
{
	ERROR("Not implemented");

	Format format = Format::LUMINANCE16;
	out.create( width, height, format);
}

void VXConvolution::load(const RawImage &in, uint32_t scale )
{
	ERROR_ON( in.format() != Format::LUMINANCE16 );
	ERROR_ON( in.width() != width );
	ERROR_ON( in.height() != height );
	uint n_elements = width * height;
	vx_int16 coeffs[n_elements];
	/* Flip the matrix, because VX is going to read it backwards: https://cvs.khronos.org/bugzilla/show_bug.cgi?id=11431 */
	for(uint i = 0; i < n_elements; ++i)
	{
		coeffs[i] = reinterpret_cast<vx_int16*>(in.data())[n_elements - 1 - i];
	}
	CHECK_VX( vxCopyConvolutionCoefficients( convolution, coeffs, VX_WRITE_ONLY, VX_MEMORY_TYPE_HOST) );
	if( scale == 0 ) {
		vx_int32 s = 0;
		vx_int16 *ptr = (vx_int16 *) in.data();
		for( vx_uint32 i = 0; i < width * height; i++)
		{
			s += ptr[i];
		}
		if(s == 0 ) s = 1;
		if( s < 0 ) s *= -1;
		scale = s;
	}
	CHECK_VX( vxSetConvolutionAttribute(convolution, VX_CONVOLUTION_SCALE, &scale, sizeof(scale)) );
}

VXConvolution::~VXConvolution()
{
	CHECK_VX( vxReleaseConvolution( &convolution ) );
}
