#include <mali_cv/Error.h>
#include <mali_cv/Context.h>
#include <mali_cv/Distribution.h>

using namespace mali_cv;

VXDistribution::VXDistribution():IDistribution(), dist(nullptr)
{
}

VXDistribution::~VXDistribution()
{
	CHECK_VX( vxReleaseDistribution( &dist ) );
}

void VXDistribution::internal_create(Context &ctx, size_t num_bins, int32_t offset, uint32_t range)
{
	vx_size vx_bins    = num_bins;
	vx_int32 vx_offset = offset;
	vx_uint32 vx_range = range;
	dist = vxCreateDistribution(ctx.vx.context(), vx_bins, vx_offset, vx_range);
}

void VXDistribution::read( std::vector<uint32_t> &out)const
{
	vx_size num_bins = 0;
	CHECK_VX( vxQueryDistribution(this->dist, VX_DISTRIBUTION_BINS , &num_bins, sizeof(num_bins)) );

	vx_map_id map_id;
	void* data = nullptr;
	CHECK_VX( vxMapDistribution(this->dist, &map_id, &data, VX_READ_ONLY, VX_MEMORY_TYPE_HOST, 0) );

	for( vx_size i = 0; i < num_bins; ++i )
	{
		uint32_t d = static_cast<vx_uint32*>(data)[i];
		out.push_back(d);
	}

	CHECK_VX( vxUnmapDistribution(dist, map_id) );
}
