#include "mali_cv/Pyramid.h"

#include "mali_cv/Error.h"
#include "mali_cv/Context.h"
#include "mali_cv/Image.h"

using namespace mali_cv;

VXPyramid::VXPyramid() : pyramid(nullptr)
{
}

void VXPyramid::internal_create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format fmt )
{
	switch(fmt)
	{
		case Format::U8:
			pyramid = vxCreatePyramid(ctx.vx.context(), levels, 0.5f, w, h, VX_DF_IMAGE_U8);
			break;
		case Format::S16:
			pyramid = vxCreatePyramid(ctx.vx.context(), levels, 0.5f, w, h, VX_DF_IMAGE_S16);
			break;
		default:
			ERROR_ON("Not implemented");
			break;
	}
	ERROR_ON(pyramid == NULL);
}

void VXPyramid::store( std::vector<ImagePtr> &out )
{
	size_t n_levels = 0;
	CHECK_VX( vxQueryPyramid(pyramid, VX_PYRAMID_LEVELS, &n_levels, sizeof(n_levels)) );

	ERROR_ON(out.size() != n_levels );

	for(uint i = 0; i < n_levels; i++)
	{
		/* The memory was pre-allocated, we need to release before replacing the image: */
		CHECK_VX( vxReleaseImage( &out[i]->vx.img ) );
		out[i]->vx.img = vxGetPyramidLevel(pyramid, i);
		ERROR_ON( out[i]->vx.img == nullptr);
	}
}

VXPyramid::~VXPyramid()
{
	CHECK_VX( vxReleasePyramid( &pyramid ) );
}
