#ifndef __CONTEXT_H__
#define __CONTEXT_H__

#include "mali_cv/Types.h"

#include <map>

namespace mali_cv
{
class IContext
{
	public:
		virtual ~IContext() = default;
};

#ifdef USE_OPENVX
class VXContext: public IContext
{
	public:
		VXContext();
		VXContext(const VXContext&) = delete;
		const VXContext& operator=(const VXContext&) = delete;
		~VXContext();

		void run_graph(vx_graph graph, uint num_iterations, const char* tag);
		vx_graph create_graph();
		void set_border_mode( vx_node node, BorderMode border_mode );
		vx_context context();
		vx_border_t vx_border_mode(BorderMode border_mode)const;
	private:
		vx_context _ctx;
};
#else/* USE_OPENVX */
using VXContext = IContext;
#endif /* USE_OPENVX */

class Context final
{
	public:
		Context(uint num_iterations, BorderMode mode, uint8_t constant_border_value = CONSTANT_BORDER_VALUE);
		uint num_iterations() const;
		BorderMode border_mode() const;
		uint8_t constant_border_value() const;

		VXContext vx;
	private:
		uint _num_iterations;
		BorderMode _border_mode;
		uint8_t _constant_border_value;
};
}
#endif /* __CONTEXT_H__ */
