#ifndef __CV_CONVOLUTION_H__
#define __CV_CONVOLUTION_H__

#include "mali_cv/Image.h"
#include "mali_cv/Types.h"

#include <memory>

namespace mali_cv
{
class IConvolution
{
	public:
		IConvolution();
		virtual ~IConvolution() = default;
		void create( Context &ctx, const RawImage &in, CopyData copy);
		void create( Context &ctx, size_t width, size_t height, Format format);
		virtual void load(const RawImage &in, uint32_t scale )
		{
		}
		virtual void store(RawImage &out)
		{
		}
	protected:
		virtual void internal_create( Context &ctx, size_t w, size_t h, Format format)
		{
		}
	public:
		size_t width, height;
		Format format;
};

#ifdef USE_OPENVX
class VXConvolution: public IConvolution
{
	public:
		VXConvolution();
		~VXConvolution();
		VXConvolution(const VXConvolution&)=delete;
		const VXConvolution& operator=(const VXConvolution&)=delete;
		void store( RawImage &out)override;
		void load(const RawImage &in, uint32_t scale)override;
	protected:
		void internal_create( Context &ctx, size_t w, size_t h, Format format)override;
	public:
		vx_convolution convolution;
};

#else /* USE_OPENVX */
using VXConvolution = IConvolution;
#endif /* USE_OPENVX */

#ifdef USE_OPENCV
class CVConvolution: public IConvolution
{
	public:
		CVConvolution();
		void store( RawImage &out )override;
		void load(const RawImage &in,uint32_t scale)override;
	protected:
		void internal_create( Context &ctx, size_t w, size_t h, Format format)override;
	public:
		int type;
		cv::Mat convolution;
		uint32_t _scale;
};
#else /* USE_OPENCV */
using CVConvolution = IConvolution;
#endif /* USE_OPENCV */

#ifdef USE_NEON
class NEConvolution: public IConvolution
{
	public:
		NEConvolution();
		NEConvolution(const NEConvolution&)=delete;
		const NEConvolution& operator=(const NEConvolution&)=delete;
		void store( RawImage &out )override;
		void load(const RawImage &in,uint32_t scale  )override;
	protected:
		void internal_create( Context &ctx, size_t w, size_t h, Format format)override;
	public:
		std::unique_ptr<int16_t[]> convolution;
		int16_t _scale;

};
#else /* USE_NEON */
using NEConvolution = IConvolution;
#endif /* USE_NEON */


#ifdef USE_OPENCL
class CLConvolution: public IConvolution
{
	public:
		CLConvolution();
		void store( RawImage &out )override;
		void load(const RawImage &in,uint32_t scale)override;
	protected:
		void internal_create( Context &ctx, size_t w, size_t h, Format format)override;
	public:
		std::unique_ptr<int16_t[]> convolution;
		cl_short _scale;
};
#else /* USE_OPENCL */
using CLConvolution = IConvolution;
#endif /* USE_OPENCL */

class Convolution final
{
public:
	Convolution( Context &ctx, const RawImage &in, CopyData copy);
	Convolution( Context &ctx, size_t width, size_t height, Format format = Format::LUMINANCE16);
	void load(const RawImage &in, uint32_t scale  );

	CVConvolution cv;
	VXConvolution vx;
	CLConvolution cl;
	NEConvolution ne;
};
}
#endif /* __CV_CONVOLUTION_H__ */
