#ifndef __EIGEN_FUNCTIONS_H__
#define __EIGEN_FUNCTIONS_H__

#include "mali_cv/Tensor.h"
#include "mali_cv/Timer.h"
#include "mali_cv/Types.h"
#include <vector>

namespace mali_cv
{
class EigenFunctions
{
	public:
		EigenFunctions( Context & ctx, const std::string & target_name);
		void activation_layer(EigenTensor &in, EigenTensor &out, ActivationLayerInfo act_info);
		void batchnormalization_layer(EigenTensor &a, EigenTensor &b, EigenTensor &mean, EigenTensor &var, EigenTensor &beta, EigenTensor &gamma, float epsilon);
		void pooling_layer(EigenTensor &in, EigenTensor &out, PoolingLayerInfo pool_info);
		void softmax_layer(EigenTensor &in, EigenTensor &out);
		void tensor_ops(EigenTensor &in1, EigenTensor &in2, EigenTensor &out, TensorOp op);
		void convolution_layer(EigenTensor &in, EigenTensor &w, EigenTensor *b, EigenTensor &out, PadStrideInfo conv_info);
		void locally_connected_layer(EigenTensor &in, EigenTensor &w, EigenTensor *b, EigenTensor &out, PadStrideInfo conv_info);
		void fully_connected_layer(EigenTensor &input, EigenTensor &weights, EigenTensor* bias, EigenTensor &output);
		void gemm(EigenTensor &a, EigenTensor &b, EigenTensor &c, EigenTensor &out, float alpha, float beta);
		void concatenate_depth(std::vector<EigenTensor*> &a, EigenTensor &b);
	private:
#ifdef USE_EIGEN
		const std::string _name;
		Timer _timer;
		Context &_ctx;
#endif /* USE_EIGEN */
};
}

#endif /* __EIGEN_FUNCTIONS_H__ */
