#ifndef __CV_ERROR_H__
#define __CV_ERROR_H__

#include <cstdio>
#include <string>

#define ERROR( ... ) ::mali_cv::error( __FUNCTION__, __FILE__,__LINE__, __VA_ARGS__)
#define ERROR_ON_MSG( cond, ...)\
	if( cond) ERROR(__VA_ARGS__)
#define ERROR_ON( cond )\
	if( cond) ERROR(#cond)

// Ignore the return value of a function
#define UNUSED( expr ) static_cast<void>(expr)

#ifdef MALI_DEBUG_DESTRUCTOR
#define DEBUG_INSTRUCTOR() printf("%s (this=%p)\n",__FUNCTION__,this)
#define DEBUG_DESTRUCTOR() printf("%s (this=%p)\n",__FUNCTION__,this)
#else /* MALI_DEBUG_DESTRUCTOR */
#define DEBUG_INSTRUCTOR()
#define DEBUG_DESTRUCTOR()
#endif /* MALI_DEBUG_DESTRUCTOR */

#define READ_CONFIG( attr, env, defaultValue )\
	do{\
		const char *str = getenv( env );\
		attr = ( str != nullptr ) ? atoi( str ) : defaultValue;\
		printf("%s = %d\n",env,(int) attr);\
	}while(0)

#define READ_CONFIG_STR( attr, env, defaultValue)\
	do{\
		const char *str = getenv( env );\
		attr = ( str != nullptr ) ? str : defaultValue;\
		printf("%s = \"%s\"\n",env, attr);\
	}while(0)

namespace mali_cv
{
[[ noreturn ]] void error( const char *function, const char *file, const int line, const char *msg, ...);
}
#endif /*__CV_ERROR_H__ */
