#ifndef __CV_HOG_H__
#define __CV_HOG_H__

#ifdef USE_OPENCL
#include "arm_compute/runtime/CL/CLHOG.h"
#endif
#include "arm_compute/runtime/HOG.h"

#include "mali_cv/Types.h"

#include <vector>

namespace mali_cv
{

class IHOG
{
	public:
		IHOG() = default;
		void create(const HOGInfo &hog_info);
		void init_descriptor(const std::vector<float> &descriptor);
		virtual ~IHOG() = default;
	protected:
		virtual void internal_create(const HOGInfo &hog_info) {}
		virtual void internal_init_descriptor(const std::vector<float> &descriptor) {}
};

#if defined( USE_OPENCV )
class CVHOG: public IHOG
{
	public:
		CVHOG();
	protected:
		void internal_create(const HOGInfo &hog_info)override;
		void internal_init_descriptor(const std::vector<float> &descriptor)override;
	public:
		cv::HOGDescriptor hog;
		cv::Size win_stride;
};
#else /* USE_OPENCV */
using CVHOG = IHOG;
#endif /* USE_OPENCV */

#ifdef USE_NEON
class NEHOG: public IHOG
{
	public:
		NEHOG();
	protected:
		void internal_create(const HOGInfo &hog_info)override;
		void internal_init_descriptor(const std::vector<float> &descriptor)override;
	public:
		arm_compute::HOG hog;
};

#else /* USE_NEON */
using NEHOG = IHOG;
#endif /* USE_NEON */

#ifdef USE_OPENCL
class CLHOG: public IHOG
{
	public:
		CLHOG();
	protected:
		void internal_create(const HOGInfo &hog_info)override;
		void internal_init_descriptor(const std::vector<float> &descriptor)override;
	public:
		arm_compute::CLHOG hog;
};

#else /* USE_CL */
using CLHOG = IHOG;
#endif /* USE_NEON */

/* Currently not supported in OpenVX */
using VXHOG = IHOG;

class HOG
{
	public:
		HOG(const HOGInfo &hog_info);
		void init_descriptor(const std::vector<float> &descriptor);
	public:
		CVHOG  cv;
		VXHOG  vx;
		CLHOG  cl;
		NEHOG  ne;
		size_t descriptor_size;
};
}
#endif /* __CV_HOG_H__ */
