#ifndef __CV_PYRAMID_H__
#define __CV_PYRAMID_H__

#ifdef USE_NEON
#include "arm_compute/runtime/Pyramid.h"
#endif

#ifdef USE_OPENCL
#include "arm_compute/runtime/CL/CLPyramid.h"
#endif

#include "mali_cv/Types.h"

#include <memory>
#include <vector>

namespace mali_cv
{
class IPyramid
{
	public:
		void create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format fmt );
		virtual void store( std::vector<ImagePtr> &out )
		{
		}
		virtual ~IPyramid() = default;
	protected:
		virtual void internal_create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format format )
		{
		}
};

#ifdef USE_OPENVX
class VXPyramid: public IPyramid
{
	public:
		VXPyramid();
		~VXPyramid();
		VXPyramid(const VXPyramid&)=delete;
		const VXPyramid& operator=(const VXPyramid&)=delete;
		vx_df_image to_vx_format(  Format format  );

		void store( std::vector<ImagePtr> &in )override;
	protected:
		void internal_create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format format )override;
	public:
		vx_pyramid pyramid;
};

#else /* USE_OPENVX */
using VXPyramid = IPyramid;
#endif /* USE_OPENVX */

#ifdef USE_OPENCV
class CVPyramid: public IPyramid
{
	public:
		CVPyramid();
		void store( std::vector<ImagePtr> &in )override;
	protected:
		void internal_create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format fmt )override;
	public:
		std::vector<CVImagePtr> pyramid;
};
#else /* USE_OPENCV */
using CVPyramid = IPyramid;
#endif /* USE_OPENCV */

#ifdef USE_NEON
class NEPyramid: public IPyramid
{
	public:
		NEPyramid();
		void store( std::vector<ImagePtr> &in )override;
	protected:
		void internal_create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format fmt )override;
	public:
		arm_compute::Pyramid pyramid;
};

#else /* USE_NEON */
using NEPyramid = IPyramid;
#endif /* USE_NEON */

#ifdef USE_OPENCL
class CLPyramid: public IPyramid
{
	public:
		CLPyramid();
		void store( std::vector<ImagePtr> &in )override;
	protected:
		void internal_create( Context &ctx, size_t levels, float scale, size_t w, size_t h, Format fmt )override;
	public:
		arm_compute::CLPyramid pyramid;
};
#else /* USE_OPENCL */
using CLPyramid = IPyramid;
#endif /* USE_OPENCL */

class Pyramid final
{
public:
	Pyramid( Context &ctx, size_t levels, float scale_factor, size_t w, size_t h, Format fmt);
	void store( std::vector<ImagePtr> &out);

	CVPyramid cv;
	VXPyramid vx;
	CLPyramid cl;
	NEPyramid ne;

	size_t   n_levels;
	size_t   width;
	size_t   height;
	Format   format;
	float    scale;
	Context& context;
};
}
#endif /* __CV_PYRAMID_H__ */
