#ifndef __RAW_IMAGE_H__
#define __RAW_IMAGE_H__

#include "mali_cv/RawImageBase.h"

#include <memory>
#include <string>
#include <vector>

namespace mali_cv
{
class RawImage: public RawImageBase
{
	public:
		RawImage();
		RawImage( const RawImage &src, CopyData copy_data);
		RawImage( size_t width, size_t height, Format format);
#if 0
		void createFromBMP( const char *filename );
		void saveToBMP( const char *filename );
#endif
		void random_init( float low, float high );
		void create_diff( const RawImage &reference, float amplify = 1.0f );
		void create_from( const std::string &filename);
		void save_to( const std::string &filename);
		void create_from_ppm( const std::string  &filename);
		void create_from_pgm( const std::string  &filename);
		void save_to_ppm( const std::string &filename );
		void save_to_pgm( const std::string &filename );
		void create_from_txt( const std::string &filename );
		void save_to_txt( const std::string &filename );
		void create_from_bin( const std::string &filename);
		void save_to_bin( const std::string &filename );
		void allocate_from_image( const RawImage &src );
		void create( size_t width, size_t height, Format fmt );
		void copy_from( const RawImage &src, Format fmt = Format::SAME_AS_SRC );
		void deinterleave( std::vector<RawImage *> &channels_out);
		void interleave( std::vector<RawImage *> &channels_in);
		unsigned char *data()const;
	private:
		void copy( size_t width, size_t height, const unsigned char *src, Format src_format, Format dst_format, size_t src_row_padding = 0);
		std::unique_ptr<unsigned char[]> _data;
};
}
#endif /* __RAW_IMAGE_H__ */
