#ifndef __CV_REMAPTABLE_H__
#define __CV_REMAPTABLE_H__

#include "mali_cv/Types.h"

#include <vector>

namespace mali_cv
{
class IRemapTable
{
	public:
		virtual ~IRemapTable() = default;
		void create(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height);
		virtual void set_remap_point(int dst_x, int dst_y, float src_x, float src_y)
		{
		}
		virtual void map()
		{
		}
		virtual void unmap()
		{
		}
	protected:
		virtual void internal_create(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height)
		{
		}
};

#ifdef USE_OPENVX
class VXRemapTable: public IRemapTable
{
	public:
		VXRemapTable();
		~VXRemapTable();
		VXRemapTable(const VXRemapTable&)=delete;
		const VXRemapTable& operator=(const VXRemapTable&)=delete;
		void set_remap_point(int dst_x, int dst_y, float src_x, float src_y)override;
	protected:
		void internal_create(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height)override;
	public:
		vx_remap table;
};
#else /* USE_OPENVX */
using VXRemapTable = IRemapTable;
#endif /* USE_OPENVX */

#ifdef USE_OPENCV
class CVRemapTable: public IRemapTable
{
	public:
		CVRemapTable();
		void set_remap_point(int dst_x, int dst_y, float src_x, float src_y)override;
	protected:
		void internal_create(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height)override;
	public:
		std::vector<CVImagePtr> table;
};
#else /* USE_OPENCV */
using CVRemapTable = IRemapTable;
#endif /* USE_OPENCV */

#ifdef USE_NEON
class NERemapTable: public IRemapTable
{
	public:
		NERemapTable();
		void set_remap_point(int dst_x, int dst_y, float src_x, float src_y)override;
	protected:
		void internal_create(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height)override;
	public:
		std::vector<NEImagePtr> table;
};
#else /* USE_NEON */
using NERemapTable = IRemapTable;
#endif /* USE_NEON */

#ifdef USE_OPENCL
class CLRemapTable: public IRemapTable
{
	public:
		CLRemapTable();
		void set_remap_point(int dst_x, int dst_y, float src_x, float src_y)override;
		void map() override;
		void unmap() override;
	protected:
		void internal_create(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height)override;
	public:
		std::vector<CLImagePtr> table;
};
#else /* USE_OPENCL */
using CLRemapTable = IRemapTable;
#endif /* USE_OPENCL */

class RemapTable final
{
public:
	RemapTable(Context &ctx, size_t src_width, size_t src_height, size_t dst_width, size_t dst_height);
	void set_remap_point(int dst_x, int dst_y, float src_x, float src_y);
	void map();
	void unmap();

	CVRemapTable cv;
	VXRemapTable vx;
	CLRemapTable cl;
	NERemapTable ne;

	size_t src_width;
	size_t src_height;
	size_t dst_width;
	size_t dst_height;
	Context &context;
};
}
#endif /* __CV_REMAPTABLE_H__ */
