#ifndef __CV_SCALAR_H__
#define __CV_SCALAR_H__

namespace mali_cv
{

template <class T>
class IScalar
{
	public:
		IScalar<T>():scalar(){};
		virtual ~IScalar<T>() {};
		virtual void internal_init( T val ) { scalar = val; }
		virtual T read() { return scalar; }
	public:
		T scalar;
};

#if defined( USE_OPENCV )
template <class T>
class CVScalar : IScalar<T>
{
	public:
		CVScalar() : scalar() {};
		virtual void internal_init( T val ) { scalar.val[0] = val; }
		virtual T read() { return scalar.val[0]; }
	public:
		cv::Scalar scalar;
};
#else
template <class T>
using CVScalar = IScalar<T>;
#endif

/* C++11 feature: added alias declarations, which is the generalization of typedef */
/* Typedef does not allow template */
template <class T>
using VXScalar = IScalar<T>;
template <class T>
using CLScalar = IScalar<T>;
template <class T>
using NEScalar = IScalar<T>;

template <class T>
class Scalar
{
	public:
		Scalar();
		Scalar(T scalar):cv(),vx(),cl(),ne() { init(scalar); }
		void init(T scalar)
		{
			cv.internal_init(scalar);
			vx.internal_init(scalar);
			cl.internal_init(scalar);
			ne.internal_init(scalar);
		}
	public:
		CVScalar<T> cv;
		VXScalar<T> vx;
		CLScalar<T> cl;
		NEScalar<T> ne;
};

}

#endif /* __CV_IMAGE_H__ */
