#ifndef __TIMER_H__
#define __TIMER_H__

#include <chrono>
#include <string>

#if ARM_CV_ENABLE_PMU
#include <linux/perf_event.h>
#include <linux/hw_breakpoint.h>
#endif

namespace mali_cv
{
class Timer final
{
	public:
		Timer(const std::string &msg = "");
		void start(void);
		std::chrono::high_resolution_clock::duration stop(void);
		void reset(void);
		long long get_cycles(bool reset = false);
		float get_elapsed_ms(bool reset = false);
		void print_elapsed_ms(bool reset = false);
	private:
		std::chrono::high_resolution_clock::time_point _start;
		std::chrono::high_resolution_clock::time_point _stop;
		std::string _msg;
		static const std::chrono::high_resolution_clock::time_point ZERO_TIME;
#if ARM_CV_ENABLE_PMU
		struct perf_event_attr _pe;
		long _fd_instr,_fd_cycles;
		long long _inst_count, _cycles_count;
#endif


};
}
#endif /* __TIMER_H__ */
