#ifndef __CVTYPES_H__
#define __CVTYPES_H__

#ifdef USE_OPENVX
#include <VX/vx.h>
#include <VX/vxu.h>
#endif /* USE_OPENVX */

#ifdef USE_OPENCV
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Weffc++"
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/features2d.hpp>
#include <opencv2/video/tracking.hpp>
#include <opencv2/objdetect.hpp>
#pragma GCC diagnostic pop
#endif /* USE_OPENCV */

#ifdef USE_NEON
#include <arm_neon.h>
#endif /* USE_NEON */

#ifdef USE_OPENVX
#define CHECK_VX( func )\
	ERROR_ON( (func) != VX_SUCCESS)
#endif /* USE_OPENVX */

#include "arm_compute/core/HOGInfo.h"
#include "arm_compute/core/Size2D.h"
#include "arm_compute/core/TensorInfo.h"
#include "arm_compute/core/Types.h"
#include "arm_compute/core/Coordinates.h"
#include "arm_compute/core/TensorShape.h"

#include <memory>

#define DECLARE_SHARED_POINTER_CLASS( type )\
	class type;\
	typedef std::shared_ptr<type> type ## Ptr;\
	template<class ...Types> type ## Ptr type ## Alloc(Types ... args)\
	{\
		return std::make_shared<type>(args...);\
	}

namespace mali_cv
{
/* Constant value used to indicate maximum dimension of a Tensor */
const size_t MAX_DIMS = arm_compute::MAX_DIMS;

const uint8_t CONSTANT_BORDER_VALUE = 199;

/* Constant value used to indicate a half-scale pyramid */
const float SCALE_PYRAMID_HALF = arm_compute::SCALE_PYRAMID_HALF;

/* Constant value used to indicate a ORB scaled pyramid */
const float SCALE_PYRAMID_ORB = arm_compute::SCALE_PYRAMID_ORB;

using float16_t = arm_compute::half;

using BorderMode = arm_compute::BorderMode;
using InterpolationMode = arm_compute::InterpolationPolicy;
using Channel = arm_compute::Channel;
using DataType = arm_compute::DataType;
using Size2D = arm_compute::Size2D;
using MatrixPattern = arm_compute::MatrixPattern;
using NonLinearFilterFunction = arm_compute::NonLinearFilterFunction;
using NormType = arm_compute::NormType;

class ITarget;

DECLARE_SHARED_POINTER_CLASS( Image )

#ifdef USE_OPENCL
DECLARE_SHARED_POINTER_CLASS( CLImage )
#endif /*USE_OPENCL */

#ifdef USE_NEON
DECLARE_SHARED_POINTER_CLASS( NEImage )
#endif /*USE_NEON*/

#ifdef USE_OPENCV
DECLARE_SHARED_POINTER_CLASS( CVImage )
#endif /*USE_OPENCV */

DECLARE_SHARED_POINTER_CLASS( Convolution )
DECLARE_SHARED_POINTER_CLASS( Context )
DECLARE_SHARED_POINTER_CLASS( Pyramid )
DECLARE_SHARED_POINTER_CLASS( RemapTable )

enum class CopyData { YES = 1, NO = 0 };
/*

    NV12 : 2-plane, UV interleaved
    NV21 : 2-plane, VU interleaved
    UYVY : 1-plane, U0, Y0, V0, Y1
    YUYV : 1-plane, Y0, U0, Y1, V0
    IYUV : 3-plane, YUV420
    YUV4 : 3-plane, YUV444

*/
enum class Format
{
	UNKNOWN     = 0,
	SAME_AS_SRC,
	RGB888,
	YUV444,
	YUYV422,
	LUMINANCE,
	U8 = LUMINANCE,
	S8,
	RGBA8888,
	RGBA_FLOAT,
	RGBA16,
	LUMINANCE16,
	S16 = LUMINANCE16,
	U16,
	LUMINANCE32,
	S32 = LUMINANCE32,
	U32,
	NV12,
	NV21,
	UYVY,
	IYUV,
	FP16C1,
	FP32C1,
	FP16C2,
	FP32C2,
	NUM_FORMATS
};

/** Tensor Operations */
enum class TensorOp
{
    ADD,
    SUB,
    MUL,
    LUT,
    TRANSPOSE
};

using IKeyPoint = arm_compute::KeyPoint;
using ICoordinates2D = arm_compute::Coordinates2D;
using ConvertPolicy = arm_compute::ConvertPolicy;
using RoundingPolicy = arm_compute::RoundingPolicy;
using Termination = arm_compute::Termination;
using ActivationLayerInfo = arm_compute::ActivationLayerInfo;
using ActivationFunction = arm_compute::ActivationLayerInfo::ActivationFunction;
using PoolingLayerInfo = arm_compute::PoolingLayerInfo;
using PoolingType = arm_compute::PoolingType;
using PadStrideInfo = arm_compute::PadStrideInfo;
using DimensionRoundingType = arm_compute::DimensionRoundingType;
using NormalizationLayerInfo = arm_compute::NormalizationLayerInfo;
using TensorShape = arm_compute::TensorShape;
using TensorInfo = arm_compute::TensorInfo;
using HOGInfo = arm_compute::HOGInfo;
using HOGNormType = arm_compute::HOGNormType;
}
#endif /* __CVTYPES_H__ */
