#ifndef __CV_VALIDATE_H__
#define __CV_VALIDATE_H__

#include "mali_cv/Array.h"
#include "mali_cv/Distribution.h"
#include "mali_cv/Image.h"
#include "mali_cv/RawImage.h"
#include "mali_cv/Scalar.h"
#include "mali_cv/Tensor.h"
#include "mali_cv/Types.h"

#include <utility>

namespace mali_cv
{
class Environment;
class KeyPointArray;
class Environment;

class Validate
{
	public:
		/** Compare two raw images. Only elements within the specified valid
		 *  region will be compared.
		 *
		 * @return Number of mismatching elements.
		 */
		static unsigned int compare(const RawImage &img1, const RawImage &img2, const arm_compute::ValidRegion &valid_region,
		                            float threshold = 0.0f, float acceptable_ratio_of_mismatches = 0.0f, bool print_all = false);

		/** Compare two raw images.
		 *
		 * @return Number of mismatching elements.
		 */
		static unsigned int compare(const RawImage &img1, const RawImage &img2,
		                            float threshold = 0.0f, float acceptable_ratio_of_mismatches = 0.0f, bool print_all = false);

		/** Compare two valid regions.
		 *
		 * @return True if the regions match, false otherwise.
		 */
		static bool compare(const arm_compute::ValidRegion &region1, const arm_compute::ValidRegion &region2);

		static void single_image_output(const Environment &env, Image &img, const RawImage &ref, const arm_compute::ValidRegion &valid_region, float threshold_f);
		static void single_image_output(const Environment &env, Image &img);
		static void single_image_output(const Environment &env, Image &img, const arm_compute::ValidRegion &valid_region);
		static void single_image_output(const Environment &env, Image &img, float ref_lo, float ref_hi, const arm_compute::ValidRegion &valid_region, float threshold_f);

		static void double_image_output(const Environment &env, Image &img1, Image &img2);
		static void double_image_output(const Environment &env, Image &img1, Image &img2, const arm_compute::ValidRegion &valid_region);

		static void single_tensor_output(const Environment &env, Tensor &tensor, float threshold);

		static void single_image_output( Image &img, const Environment &env, uint border_width = 0) /*__attribute__((deprecated("Use new function!")))*/;
		static void single_image_output( Image &img, const RawImage &ref, float threshold_f, const Environment &env, uint border_width = 0) /*__attribute__((deprecated("Use new function!")))*/;
		static void double_image_output( Image &img1, Image &img2, const Environment &env, uint border_width=0) /*__attribute__((deprecated("Use new function!")))*/;
		static void single_tensor_output( Tensor &tensor, const Environment &env, uint border_width = 0, float custom_threshold = 0.f) /*__attribute__((deprecated("Use new function!")))*/;

		static void corners_output(const IKeyPointArray &corners, const Environment &env, int32_t threshold_distance, float threshold_strength, std::pair<unsigned int, unsigned int> image_size, unsigned int border);
		static void coords2D_output( const ICoord2DArray &coords, int32_t num_coords, const Environment &env);
		static void distribution_output(const Distribution& dist, const Environment &env);

		template <class T> static void single_scalar_output( Scalar<T> &scalar, const Environment &env );
		template <class T> static void double_scalar_output( Scalar<T> &scalar1, Scalar<T> &scalar2, const Environment &env);
	};
}
#endif /* __CV_VALIDATE_H__ */
