#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{
class Accumulate: public ITest
{
	public:
		const std::string name() const override
		{
			return "Accumulate";
		}
		Accumulate( Environment &env, RawImage &in, RawImage &out) : ITest(env), a(env.ctx, in, CopyData::YES),
			c(env.ctx, out, CopyData::YES)
		{
			ERROR_ON(in.format() != Format::LUMINANCE);
			ERROR_ON(out.format() != Format::LUMINANCE16);
		}

		virtual void run(ITarget *target)override
		{
			target->accumulate(a, c);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a, c;
};

class AccumulateSquared: public ITest
{
	public:
		const std::string name() const override
		{
			return "AccumulateSquared";
		}
		AccumulateSquared( Environment &env, RawImage &in, RawImage &out, uint32_t in_shift=0) : ITest(env), a(env.ctx, in, CopyData::YES),
			c(env.ctx, out, CopyData::YES), sh(in_shift)
		{
			ERROR_ON(in.format() != Format::LUMINANCE);
			ERROR_ON(out.format() != Format::LUMINANCE16);
		}

		virtual void run(ITarget *target)override
		{
			target->accumulate_squared(a, c, sh);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a, c;
		const uint32_t sh;
};

class AccumulateWeighted: public ITest
{
	public:
		const std::string name() const override
		{
			return "AccumulateWeighted";
		}
		AccumulateWeighted( Environment &env, RawImage &in, RawImage& out, float alpha) : ITest(env), a(env.ctx, in, CopyData::YES),
			c(env.ctx, out, CopyData::YES), wh(alpha)
		{
			ERROR_ON(alpha > 1 || alpha < 0 );
			ERROR_ON(in.format() != Format::LUMINANCE);
			ERROR_ON(out.format() != Format::LUMINANCE);
		}

		virtual void run(ITarget *target)override
		{
			target->accumulate_weighted(a, c, wh);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a, c;
		const float wh;
};


class AccumulateWeighted_FP16: public AccumulateWeighted
{
	public:
		const std::string name() const override
		{
			return "AccumulateWeighted_FP16";
		}
		AccumulateWeighted_FP16( Environment &env, RawImage &in, RawImage& out, float alpha) : AccumulateWeighted(env,in,out,alpha) {}

		void run(ITarget *target)override
		{
			target->accumulate_weighted_fp16(a, c, wh);
		}
		void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
};



}
