#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{

class ArithmeticAddition: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticAddition";}

		ArithmeticAddition( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES), b(env.ctx, _b, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE),p(policy) {}

		virtual void run(ITarget *target)override
		{
			target->arithmetic_addition(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};

class ArithmeticSubtraction: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticSubtraction";}

		ArithmeticSubtraction( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES), b(env.ctx, _b, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE),p(policy) {}

		virtual void run(ITarget *target)override
		{
			target->arithmetic_subtraction(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};


class ArithmeticAdditionU16U16U16: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticAddition_U16U16U16";}

		ArithmeticAdditionU16U16U16( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES),
			b(env.ctx, _b, CopyData::YES),
			c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16), p(policy)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE16);
			ERROR_ON(_b.format() != Format::LUMINANCE16);
		}
		virtual void run(ITarget *target)override
		{
			target->arithmetic_addition(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};



class ArithmeticAdditionU16U8U16: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticAddition_U16U8U16";}

		ArithmeticAdditionU16U8U16( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES),
			b(env.ctx, _b, CopyData::YES),
			c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16), p(policy)
	{
		ERROR_ON(_a.format() != Format::LUMINANCE16);
		ERROR_ON(_b.format() != Format::LUMINANCE);
	}

		virtual void run(ITarget *target)override
		{
			target->arithmetic_addition(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};

class ArithmeticAdditionU16U16U16Saturate: public ArithmeticAdditionU16U16U16
{
	public:
		const std::string name() const override { return "ArithmeticAddition_U16U16U16_Saturate";}
		ArithmeticAdditionU16U16U16Saturate( Environment &env, RawImage &_a, RawImage &_b) : ArithmeticAdditionU16U16U16(env,_a,_b,1) {}
};

class ArithmeticAdditionU16U16U16Wrap: public ArithmeticAdditionU16U16U16
{
	public:
		const std::string name() const override { return "ArithmeticAddition_U16U16U16_Wrap";}
		ArithmeticAdditionU16U16U16Wrap( Environment &env, RawImage &_a, RawImage &_b) : ArithmeticAdditionU16U16U16(env,_a,_b,0) {}
};

class ArithmeticSubtractionU16U8U16: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticSubtraction_U16U8U16";}

		ArithmeticSubtractionU16U8U16( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES),
			b(env.ctx, _b, CopyData::YES),
			c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16), p(policy)
	{
		ERROR_ON(_a.format() != Format::LUMINANCE16);
		ERROR_ON(_b.format() != Format::LUMINANCE);
	}

		virtual void run(ITarget *target)override
		{
			target->arithmetic_subtraction(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};

class ArithmeticAdditionU8U16U16: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticAddition_U8U16U16";}

		ArithmeticAdditionU8U16U16( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES),
			b(env.ctx, _b, CopyData::YES),
			c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16), p(policy)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE);
			ERROR_ON(_b.format() != Format::LUMINANCE16);
		}

		virtual void run(ITarget *target)override
		{
			target->arithmetic_addition(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};

class ArithmeticSubtractionU8U16U16: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticSubtraction_U8U16U16";}

		ArithmeticSubtractionU8U16U16( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES),
			b(env.ctx, _b, CopyData::YES),
			c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16), p(policy)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE);
			ERROR_ON(_b.format() != Format::LUMINANCE16);
		}

		virtual void run(ITarget *target)override
		{
			target->arithmetic_subtraction(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};

class ArithmeticAdditionU8U8U16: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticAddition_U8U8U16";}

		ArithmeticAdditionU8U8U16( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES),
			b(env.ctx, _b, CopyData::YES),
			c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16), p(policy)  {}

		virtual void run(ITarget *target)override
		{
			target->arithmetic_addition(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};

class ArithmeticSubtractionU8U8U16: public ITest
{
	public:
		const std::string name() const override { return "ArithmeticSubtraction_U8U8U16";}

		ArithmeticSubtractionU8U8U16( Environment &env, RawImage &_a, RawImage &_b, uint8_t policy ) : ITest(env),
			a(env.ctx, _a, CopyData::YES),
			b(env.ctx, _b, CopyData::YES),
			c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16), p(policy)  {}

		virtual void run(ITarget *target)override
		{
			target->arithmetic_subtraction(a,b,c,p);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		const uint8_t p;
};


}


