#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace
{
	mali_cv::TensorShape convert_shape_to_vector(mali_cv::TensorShape in_shape)
	{
		mali_cv::TensorShape vector_shape;
		vector_shape.set(0, in_shape[2]);
		return vector_shape;
	}
}

namespace mali_cv
{
class BatchNormalizationLayer : public ITest
{
	public:
		const std::string name() const override
		{
			return "BatchNormalizationLayer";
		}
		BatchNormalizationLayer(Environment &env, TensorShape shape, DataType dt, float epsilon = 1e-50)
		: ITest(env),
		  _in(env.ctx, shape, dt),
		  _out(env.ctx, shape, dt),
		  _mean(env.ctx, convert_shape_to_vector(shape), dt),
		  _var(env.ctx, convert_shape_to_vector(shape), dt),
		  _beta(env.ctx, convert_shape_to_vector(shape), dt),
		  _gamma(env.ctx, convert_shape_to_vector(shape), dt),
		  _epsilon(epsilon)
		{
			_in.random_fill(-5, 5);
			_mean.random_fill(-5, 5);
			_var.random_fill(0, 10);
			_beta.random_fill(-5, 5);
			_gamma.random_fill(-5, 5);
		}

		virtual void run(ITarget *target) override
		{
			target->batchnormalization_layer(_in, _out, _mean, _var, _beta, _gamma, _epsilon);
		}
		virtual void validate_outputs() override
		{
			Validate::single_tensor_output( _out, env, 0, 1e-3 );
		}
	protected:
		Tensor _in, _out, _mean, _var, _beta, _gamma;
		float _epsilon;
};
}
