#include <mali_cv/Context.h>
#include <mali_cv/Types.h>

#include <sstream>
#include <string>

namespace mali_cv
{
class ChannelCombine: public ITest
{
	public:
		ChannelCombine(Environment &env, RawImage &p0, RawImage &p1,
		               RawImage &p2, RawImage *p3, Format format) : ITest(env),
		    _p0(env.ctx, p0, CopyData::YES),
		    _p1(env.ctx, p1, CopyData::YES),
		    _p2(env.ctx, p2, CopyData::YES),
		    _p3(env.ctx, p3, CopyData::YES),
		    _out(env.ctx, p0.width(), p0.height(), format),
		    _name()
		{
			std::stringstream tmp_name;
			tmp_name << "ChannelCombine_"<< RawImageBase::format_str(format);
			_name = tmp_name.str();
		}
		void run(ITarget *target) override
		{
			target->channel_combine(_p0, _p1, _p2, _p3, _out);
		}
		void validate_outputs() override
		{
			Validate::single_image_output(_out, env);
		}
		const std::string name() const override
		{
			return _name;
		}
	protected:
		Image _p0, _p1, _p2, _p3, _out;
		std::string _name;
};
}
