#include <mali_cv/Types.h>
#include <mali_cv/Context.h>

#include <cstring>
#include <sstream>

namespace mali_cv
{

static size_t calculate_width_from_channel( RawImage &in, Channel channel )
{
	size_t width = in.width();
	Format fmt = in.format();
	switch( fmt )
	{
		case Format::RGB888:
		case Format::RGBA8888:
		case Format::YUV444:
			break;
		case Format::IYUV:
		case Format::YUYV422:
		case Format::NV12:
		case Format::NV21:
		case Format::UYVY:
			if (Channel::Y != channel)
				width /= 2;
			break;
		default:
			ERROR("Not supported format-channel combination.");
			break;
	}
	return width;
}

static size_t calculate_height_from_channel( RawImage &in, Channel channel )
{
	size_t height = in.height();
	Format fmt = in.format();
	switch( fmt )
	{
		case Format::RGB888:
		case Format::RGBA8888:
		case Format::YUV444:
		case Format::YUYV422:
		case Format::UYVY:
			break;
		case Format::NV12:
		case Format::NV21:
		case Format::IYUV:
			if (Channel::Y != channel)
				height /= 2;
			break;
		default:
			ERROR("Not supported format-channel combination.");
			break;
	}
	return height;
}


class ChannelExtract: public ITest
{
	public:
		const std::string name() const override
		{
			std::ostringstream s;
			s << "ChannelExtract_" << RawImageBase::format_str(a.vx.format);
			return s.str();
		}
		ChannelExtract( Environment &env, RawImage &_a, Channel _channel)
		: ITest(env), a(env.ctx, _a, CopyData::YES),
		  c(env.ctx, calculate_width_from_channel(_a, _channel), calculate_height_from_channel(_a, _channel), Format::LUMINANCE),
		  channel(_channel)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->channel_extract(a, channel, c);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env );
		}
	protected:
		Image a, c;
		Channel channel;
};


}


