#include "mali_cv/ITest.h"
#include "mali_cv/Context.h"
#include "mali_cv/Types.h"
#include "arm_compute/core/Utils.h"

#include <vector>
#include <memory>
#include <algorithm>

namespace
{
TensorShape calculate_output_shape(std::vector<TensorShape> input_shapes)
{
	TensorShape out_shape = input_shapes.at(0);

	unsigned int max_x = 0;
	unsigned int max_y = 0;
	unsigned int depth = 0;

	for(auto const& shape: input_shapes)
	{
		max_x = std::max<unsigned int>(shape.x(), max_x);
		max_y = std::max<unsigned int>(shape.y(), max_y);
		depth += shape.z();
	}

	out_shape.set(0, max_x);
	out_shape.set(1, max_y);
	out_shape.set(2, depth);

	return out_shape;
}
}

namespace mali_cv
{

class DepthConcatenate: public ITest
{
	public:
		const std::string name() const override
		{
			return "DepthConcatenate";
		}

		DepthConcatenate(Environment &env, std::vector<TensorShape>&& input_shapes )
		:ITest(env),
		  _inputs_vector(),
		  _output_shape( calculate_output_shape(input_shapes)),
		  _output(env.ctx, _output_shape, DataType::F32)
		{
			for(auto shape : input_shapes)
			{
				std::unique_ptr<Tensor> tmp( new Tensor(env.ctx, shape, DataType::F32));

				tmp->random_fill(-1, 1);

				_inputs_vector.push_back(std::move(tmp));
			}
		}

		virtual void run(ITarget *target) override
		{
			target->concatenate_depth(_inputs_vector, _output);
		}

		virtual void validate_outputs() override
		{
			Validate::single_tensor_output( _output, env );
		}

	private:
		std::vector<std::unique_ptr<Tensor>>   _inputs_vector;
		TensorShape _output_shape;
		Tensor _output;
};

}
