#include <cstring>
#include <mali_cv/Types.h>
#include <mali_cv/Context.h>

namespace mali_cv
{
class DepthConvertBase: public ITest
{
	public:
		const std::string name() const override
		{
			return "DepthConvertBase";
		}
		DepthConvertBase( Environment &env, RawImage &in, Format  fmt, ConvertPolicy policy , uint32_t shift) : 
			ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), fmt), p(policy), s(shift)
		{
		}

		virtual void run(ITarget *target)override
		{
			target->convert_depth(a, c, p, s );
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a, c;
		ConvertPolicy p;
		uint32_t s;
};

class DepthConvert_U8_U16 : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_U8_U16";
		}
		DepthConvert_U8_U16( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in,Format::U16,ConvertPolicy::SATURATE,shift)
		{
			ERROR_ON(in.format() != Format::U8);
		}
};

class DepthConvert_U8_S16 : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_U8_S16";
		}
		DepthConvert_U8_S16( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in,Format::S16,ConvertPolicy::SATURATE,shift)
		{
			ERROR_ON(in.format() != Format::U8);
		}
};

class DepthConvert_U8_S32 : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_U8_S32";
		}
		DepthConvert_U8_S32( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in, Format::S32 , ConvertPolicy::SATURATE,shift)
		{
			ERROR_ON(in.format() != Format::U8);
		}
};

class DepthConvert_U8_U32 : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_U8_U32";
		}
		DepthConvert_U8_U32( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in,Format::U32,ConvertPolicy::SATURATE,shift)
		{
			ERROR_ON(in.format() != Format::U8);
		}
};


class DepthConvert_S16_S32 : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_S16_S32";
		}
		DepthConvert_S16_S32( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in, Format::S32 , ConvertPolicy::SATURATE,shift)
		{
			ERROR_ON(in.format() != Format::S16);
		}
};

class DepthConvert_U16_U32 : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_U16_U32";
		}
		DepthConvert_U16_U32( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in, Format::U32 , ConvertPolicy::SATURATE,shift)
		{
			ERROR_ON(in.format() != Format::U16);
		}
};

class DepthConvert_U16_U8_SATURATE : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_U16_U8_SATURATE";
		}
		DepthConvert_U16_U8_SATURATE( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in, Format::U8 , ConvertPolicy::SATURATE,shift)
		{
			ERROR_ON(in.format() != Format::U16);
		}
};

class DepthConvert_U16_U8_WRAP : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_U16_U8_WRAP";
		}
		DepthConvert_U16_U8_WRAP( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in, Format::U8 , ConvertPolicy::WRAP,shift)
		{
			ERROR_ON(in.format() != Format::U16);
		}
};

class DepthConvert_S16_U8_SATURATE : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_S16_U8_SATURATE";
		}
		DepthConvert_S16_U8_SATURATE( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in,Format::U8,ConvertPolicy::SATURATE,shift)
		{
			ERROR_ON(in.format() != Format::S16);
		}
};

class DepthConvert_S16_U8_WRAP : public DepthConvertBase
{
	public:
		const std::string name() const override
		{
			return "DepthConvert_S16_U8_WRAP";
		}
		DepthConvert_S16_U8_WRAP( Environment &env, RawImage &in, uint32_t shift = 0) : DepthConvertBase(env,in,Format::U8,ConvertPolicy::WRAP,shift)
		{
			ERROR_ON(in.format() != Format::S16);
		}
};

}


