#include "mali_cv/Array.h"
#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{
class FastCorners: public ITest
{
	public:
		const std::string name() const override
		{
			return "FastCorners";
		}

		static const size_t ResultMaxPoints = MAX_NUM_VALUES ;

		FastCorners( Environment &env, RawImage &_a, float _threshold, bool _nonmax_sup, size_t _corner_threshold) : ITest(env),
			a(env.ctx, _a, CopyData::YES),corners(env.ctx,ResultMaxPoints), num_corners(0),
			threshold(_threshold),corner_threshold(_corner_threshold),nonmax_suppression(_nonmax_sup)
		{
			ERROR_ON(_threshold < 1 && _threshold > 255 );
		}

		void run(ITarget *target)override
		{
			target->fast_corners(a,threshold,nonmax_suppression, corners, num_corners);
		}
		void validate_outputs() override
		{
			// Border consist of undefined 3 pixels in fast corners and 1 in non max suppression (if run)
			const unsigned int border = 3 + (nonmax_suppression ? 1 : 0);
			Validate::corners_output( corners, env, corner_threshold, 0.1f, std::make_pair(a.width, a.height), border);
		}
	protected:
		Image a;
		IKeyPointArray corners;
		size_t num_corners;
		const float threshold;
		const size_t corner_threshold;
		const bool nonmax_suppression;
};

class FastCornersNonMaxSuppressed: public FastCorners
{
	public:
		const std::string name() const override
		{
			return "FastCornersNonMaxSuppressed";
		}
		FastCornersNonMaxSuppressed( Environment &env, RawImage &_a, float _threshold, size_t corner_threshold) : FastCorners(env,_a,_threshold,true, corner_threshold) {}
};
}
