#include "mali_cv/Array.h"
#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{
class HarrisCorners: public ITest
{
	public:
		const std::string name() const override
		{
			return "HarrisCorners";
		}
		HarrisCorners( Environment &env, RawImage &_a, float _threshold,
				float _min_dist, float _sensitivity,uint8_t _gradient, uint8_t _block_size, size_t _corner_threshold) : ITest(env),
			a(env.ctx, _a, CopyData::YES),corners(env.ctx,MAX_NUM_VALUES), num_corners(0),
			threshold(_threshold), corner_threshold(_corner_threshold), min_dist(_min_dist),sensitivity(_sensitivity),
			gradient(_gradient),block_size(_block_size) {}

		virtual void run(ITarget *target)override
		{
			target->harris_corners(a,threshold,min_dist,sensitivity,gradient,block_size, corners, num_corners);
		}
		virtual void validate_outputs() override
		{
			// Border consist of undefined pixels sobel + undefined pixels harris score + 1 undefined pixel non max suppression
			// + OpenVX sample implementation adds one more undefined pixel for no apparent reason
			const unsigned int border = gradient / 2 + block_size / 2 + 1 + 1;
			Validate::corners_output( corners, env, corner_threshold, 0.1f, std::make_pair(a.width, a.height), border);
		}
	protected:
		Image a;
		IKeyPointArray corners;
		size_t num_corners;
		const float threshold;
		size_t corner_threshold;
		const float min_dist;
		const float sensitivity;
		const uint8_t gradient;
		const uint8_t block_size;
};

class HarrisCorners_FP16: public HarrisCorners
{
	public:
		const std::string name() const override
		{
			return "HarrisCorners FP16";
		}
		HarrisCorners_FP16( Environment &env, RawImage &_a, float _threshold, float _min_dist, float _sensitivity,uint8_t _gradient, uint8_t _block_size, size_t _corner_threshold) :
					HarrisCorners(env,_a,_threshold,_min_dist,_sensitivity,_gradient,_block_size,_corner_threshold) {}
		void run(ITarget *target)override
		{
			target->harris_corners_fp16(a,threshold,min_dist,sensitivity,gradient,block_size, corners, num_corners);
		}
};

}
