#ifndef HISTOGRAM_H
#define HISTOGRAM_H

#include <mali_cv/Types.h>
#include <cstring>

namespace mali_cv
{

class Histogram: public ITest
{
	public:
		const std::string name() const override
		{
			return "Histogram";
		}
		Histogram(Environment &env, RawImage &in, size_t num_bins, int32_t offset, uint32_t range)
		: ITest(env), a(env.ctx, in, CopyData::YES), b(env.ctx, num_bins, offset, range)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->histogram(a, b);
		}
		virtual void validate_outputs() override
		{
			Validate::distribution_output( b, env );
		}
	protected:
		Image a;
		Distribution b;
};

class EqualizeHistogram: public ITest
{
	public:
		const std::string name() const override
		{
			return "EqualizeHistogram";
		}
		EqualizeHistogram(Environment &env, RawImage &_a)
		: ITest(env), a(env.ctx, _a, CopyData::YES), b(env.ctx, _a.width(), _a.height(), Format::LUMINANCE)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->equalize_histogram(a, b);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( b, env, 0 );
		}
	protected:
		Image a, b;
};

} /* mali_cv */

#endif /* HISTOGRAM_H */

