#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{

class MinMaxLocation: public ITest
{
	public:
		const std::string name() const override { return "MinMaxLocation_U8"; }
		static const size_t ResultMaxPoints = MAX_NUM_VALUES;

		MinMaxLocation( Environment &env, RawImage &_a )
		: ITest(env), a(env.ctx, _a, CopyData::YES), min(0), max(0), min_loc_count(0), max_loc_count(0),
		  min_loc(env.ctx, ResultMaxPoints), max_loc(env.ctx, ResultMaxPoints)
		{
		}

		virtual void run(ITarget *target)override
		{
			target->minmaxlocation( a, min, max, &min_loc, &max_loc, &min_loc_count, &max_loc_count );
		}
		virtual void validate_outputs() override
		{
			Validate::double_scalar_output( min, max, env );
			Validate::double_scalar_output( min_loc_count, max_loc_count, env );
			Validate::coords2D_output( min_loc, min_loc_count.ne.scalar, env );
			Validate::coords2D_output( max_loc, max_loc_count.ne.scalar, env );
		}
	protected:
		Image a;
		Scalar<int32_t>  min;
		Scalar<int32_t>  max;
		Scalar<uint32_t> min_loc_count;
		Scalar<uint32_t> max_loc_count;
		ICoord2DArray    min_loc;
		ICoord2DArray    max_loc;
};

class MinMaxLocationS16: public ITest
{
	public:
		const std::string name() const override { return "MinMaxLocation_S16"; }
		static const size_t ResultMaxPoints = MAX_NUM_VALUES;

		MinMaxLocationS16( Environment &env, RawImage &_a )
		: ITest(env), a(env.ctx, _a, CopyData::YES), min(0), max(0), min_loc_count(0), max_loc_count(0),
		  min_loc(env.ctx, ResultMaxPoints), max_loc(env.ctx, ResultMaxPoints)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE16);
		}

		virtual void run(ITarget *target)override
		{
			target->minmaxlocation( a, min, max, &min_loc, &max_loc, &min_loc_count, &max_loc_count );
		}
		virtual void validate_outputs() override
		{
			Validate::double_scalar_output( min, max, env );
			Validate::double_scalar_output( min_loc_count, max_loc_count, env );
			Validate::coords2D_output( min_loc, min_loc_count.ne.scalar, env );
			Validate::coords2D_output( max_loc, max_loc_count.ne.scalar, env );
		}
	protected:
		Image a;
		Scalar<int32_t>  min;
		Scalar<int32_t>  max;
		Scalar<uint32_t> min_loc_count;
		Scalar<uint32_t> max_loc_count;
		ICoord2DArray    min_loc;
		ICoord2DArray    max_loc;
};

class MinMaxLocationNoLoc: public ITest
{
	public:
		const std::string name() const override { return "MinMax_U8";}

		MinMaxLocationNoLoc( Environment &env, RawImage &_a )
		: ITest(env), a(env.ctx, _a, CopyData::YES), min(0), max(0)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE);
		}

		virtual void run(ITarget *target)override
		{
			target->minmaxlocation( a, min, max );
		}
		virtual void validate_outputs() override
		{
			Validate::double_scalar_output( min, max, env );
		}
	protected:
		Image a;
		Scalar<int32_t> min;
		Scalar<int32_t> max;
};

class MinMaxLocationNoLocS16: public ITest
{
	public:
		const std::string name() const override { return "MinMax_S16";}

		MinMaxLocationNoLocS16( Environment &env, RawImage &_a )
		: ITest(env), a(env.ctx, _a, CopyData::YES), min(0), max(0)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE16);
		}

		virtual void run(ITarget *target)override
		{
			target->minmaxlocation(a, min, max);
		}
		virtual void validate_outputs() override
		{
			Validate::double_scalar_output( min, max, env );
		}
	protected:
		Image a;
		Scalar<int32_t> min;
		Scalar<int32_t> max;
};

}
