#include "mali_cv/Array.h"
#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{
class OpticalFlow: public ITest
{
	public:
		const std::string name() const override { return "OpticalFlow";}

		OpticalFlow(Environment &env, RawImage &oldImage, RawImage &newImage, IKeyPointArray* old_points, IKeyPointArray* new_keypoints_estimated,
                    Termination termination, float epsilon, uint32_t num_iterations,
                    bool use_initial_estimate, int window_dimension, size_t num_levels, size_t corner_threshold) : ITest(env),
                    a(env.ctx, oldImage, CopyData::YES),
                    b(env.ctx, newImage, CopyData::YES),
                    _new_points(env.ctx, MAX_NUM_VALUES), _old_points(old_points), _new_points_estimated(new_keypoints_estimated),
                    _termination(termination), _epsilon(epsilon), _num_iterations(num_iterations), _corner_threshold(corner_threshold),
                    _use_initial_estimate(use_initial_estimate), _window_dimension(window_dimension), _num_levels(num_levels)
		{
		}

		OpticalFlow(const OpticalFlow&) = delete;
		const OpticalFlow& operator=(const OpticalFlow&)=delete;

		virtual void run(ITarget *target)override
		{
			target->optical_flow(a, b, _old_points, _new_points_estimated, &_new_points, _termination, _epsilon, _num_iterations,
			                     _use_initial_estimate, _window_dimension, _num_levels);
		}
		virtual void validate_outputs() override
		{
			Validate::corners_output( _new_points, env, _corner_threshold, 0.1f, std::make_pair(a.width, a.height), 0);
		}
	protected:
		Image a, b;
		IKeyPointArray _new_points;
		IKeyPointArray* _old_points;
		IKeyPointArray* _new_points_estimated;
		Termination _termination;
		float _epsilon;
		uint32_t _num_iterations;
		size_t _corner_threshold;
		bool _use_initial_estimate;
		size_t _window_dimension;
		size_t _num_levels;
};
}
