#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{

class PixelWiseMultiplication: public ITest
{
	public:
		const std::string name() const override { return "PixelwiseMultiplication"; }

		PixelWiseMultiplication( Environment &env, RawImage &_a, RawImage &_b, float scale, uint8_t overflow_policy, uint8_t round_policy )
		: ITest(env), a(env.ctx, _a, CopyData::YES), b(env.ctx, _b, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE),
		  s(scale), op(overflow_policy), rp(round_policy) {}

		virtual void run(ITarget *target)override
		{
			target->pixelwise_mul(a, b, c, s, op, rp);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		float s;
		const uint8_t op, rp;
};

class PixelWiseMultiplicationU8U8S16: public ITest
{
	public:
		const std::string name() const override { return "PixelwiseMultiplication_U8U8S16";}

		PixelWiseMultiplicationU8U8S16( Environment &env, RawImage &_a, RawImage &_b, float scale, uint8_t overflow_policy, uint8_t round_policy )
		: ITest(env), a(env.ctx, _a, CopyData::YES), b(env.ctx, _b, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16),
		  s(scale), op(overflow_policy), rp(round_policy)  {}

		virtual void run(ITarget *target)override
		{
			target->pixelwise_mul(a, b, c, s, op, rp);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		float s;
		const uint8_t op, rp;
};

class PixelWiseMultiplicationU8S16S16: public ITest
{
	public:
		const std::string name() const override { return "PixelwiseMultiplication_U8S16S16";}

		PixelWiseMultiplicationU8S16S16( Environment &env, RawImage &_a, RawImage &_b, float scale, uint8_t overflow_policy, uint8_t round_policy )
		: ITest(env), a(env.ctx, _a, CopyData::YES), b(env.ctx, _b, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16),
		  s(scale), op(overflow_policy), rp(round_policy)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE);
			ERROR_ON(_b.format() != Format::LUMINANCE16);
		}

		virtual void run(ITarget *target)override
		{
			target->pixelwise_mul(a, b, c, s, op, rp);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		float s;
		const uint8_t op, rp;
};

class PixelWiseMultiplicationS16S16S16: public ITest
{
	public:
		const std::string name() const override { return "PixelwiseMultiplication_S16S16S16";}

		PixelWiseMultiplicationS16S16S16( Environment &env, RawImage &_a, RawImage &_b, float scale, uint8_t overflow_policy, uint8_t round_policy )
		: ITest(env), a(env.ctx, _a, CopyData::YES), b(env.ctx, _b, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16),
		  s(scale), op(overflow_policy), rp(round_policy)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE16);
			ERROR_ON(_b.format() != Format::LUMINANCE16);
		}

		virtual void run(ITarget *target)override
		{
			target->pixelwise_mul(a, b, c, s, op, rp);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,b,c;
		float s;
		const uint8_t op, rp;
};

}
