namespace mali_cv
{
class Remap: public ITest
{
	public:
		const std::string name() const override
		{
			return "Remap";
		}
		Remap( Environment &env, RawImage &in, RawImage &mapx, RawImage &mapy, InterpolationMode mode):ITest(env), a(env.ctx, in, CopyData::YES), d(env.ctx, in.width(), in.height(), in.format()), table(env.ctx, in.width(), in.height(), in.width(), in.height()), interpolation(mode)
		{
			ERROR_ON((mapx.format() != Format::FP32C1) || (mapy.format() != Format::FP32C1));
			ERROR_ON((mapx.width() != mapy.width()) || (mapx.height() != mapy.height()));
			ERROR_ON((in.width() != mapy.width()) || (in.height() != mapy.height()));

			const int32_t width  = mapx.width();
			const int32_t height = mapx.height();

			table.map();

			const auto mapx_ptr = reinterpret_cast<const float*>(mapx.data());
			const auto mapy_ptr = reinterpret_cast<const float*>(mapy.data());

			/* Initialize remap table */
			for(int32_t y = 0; y < height; ++y)
			{
				for(int32_t x = 0; x < width; ++x)
				{
					const float x0 = *(mapx_ptr + x + y * width);
					const float y0 = *(mapy_ptr + x + y * width);

					table.set_remap_point(x, y, x0, y0);
				}
			}

			table.unmap();
		}

		void run(ITarget *target) override
		{
			target->remap(a, table, d, interpolation);
		}

		void validate_outputs() override
		{
			Validate::single_image_output( d, env, 0 );
		}
	protected:
		Image a, d;
		RemapTable table;
		InterpolationMode interpolation;
};
} /* mali_cv */
