#ifndef __TEST_SCHARR_H__
#define __TEST_SCHARR_H__

namespace mali_cv
{

class Scharr: public ITest
{
	public:
		const std::string name() const override
		{
			return "Scharr";
		}
		Scharr( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16), d(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->scharr(a, &c, &d);
		}
		virtual void validate_outputs() override
		{
			Validate::double_image_output( c, d, env, env.border_to_exclude_if_undefined( 1 ) );
		}
	protected:
		Image a, c, d;
};

class ScharrX: public ITest
{
	public:
		const std::string name() const override
		{
			return "ScharrX";
		}
		ScharrX( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->scharr(a, &c, NULL);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, env.border_to_exclude_if_undefined( 1 ) );
		}
	protected:
		Image a, c;
};

class ScharrY: public ITest
{
	public:
		const std::string name() const override
		{
			return "ScharrY";
		}
		ScharrY( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->scharr(a, NULL, &c);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, env.border_to_exclude_if_undefined( 1 ) );
		}
	protected:
		Image a, c;
};
} /* mali_cv */
#endif /* __TEST_SCHARR_H__ */
