#ifndef __TEST_SOBEL_H__
#define __TEST_SOBEL_H__

namespace mali_cv
{

class Sobel: public ITest
{
	public:
		const std::string name() const override
		{
			return "Sobel";
		}
		Sobel( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16), d(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel(a, &c, &d);
		}
		virtual void validate_outputs() override
		{
			Validate::double_image_output( c, d, env, env.border_to_exclude_if_undefined( 1 ) );
		}
	protected:
		Image a, c, d;
};

class SobelX: public ITest
{
	public:
		const std::string name() const override
		{
			return "SobelX";
		}
		SobelX( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel(a, &c, NULL);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, env.border_to_exclude_if_undefined( 1 ) );
		}
	protected:
		Image a, c;
};

class SobelY: public ITest
{
	public:
		const std::string name() const override
		{
			return "SobelY";
		}
		SobelY( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel(a, NULL, &c);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, env.border_to_exclude_if_undefined( 1 ) );
		}
	protected:
		Image a, c;
};

class Sobel5x5: public ITest
{
	public:
		const std::string name() const override
		{
			return "Sobel5x5";
		}
		Sobel5x5( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16), d(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel5x5(a, &c, &d);
		}
		virtual void validate_outputs() override
		{
			Validate::double_image_output( c, d, env, env.border_to_exclude_if_undefined( 2 ) );
		}
	protected:
		Image a, c, d;
};

class Sobel5x5X: public ITest
{
	public:
		const std::string name() const override
		{
			return "Sobel5x5_X";
		}
		Sobel5x5X( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel5x5(a, &c, NULL);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, env.border_to_exclude_if_undefined( 2 ) );
		}
	protected:
		Image a, c;
};

class Sobel5x5Y: public ITest
{
	public:
		const std::string name() const override
		{
			return "Sobel5x5_Y";
		}
		Sobel5x5Y( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel5x5(a, NULL, &c);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, env.border_to_exclude_if_undefined( 2 ) );
		}
	protected:
		Image a, c;
};

class Sobel7x7: public ITest
{
	public:
		const std::string name() const override
		{
			return "Sobel7x7";
		}
		Sobel7x7( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16), d(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel7x7(a, &c, &d);
		}
		virtual void validate_outputs() override
		{
			Validate::double_image_output( c, d, env, env.border_to_exclude_if_undefined( 3 ) );
		}
	protected:
		Image a, c, d;
};

class Sobel7x7X: public ITest
{
	public:
		const std::string name() const override
		{
			return "Sobel7x7_X";
		}
		Sobel7x7X( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel7x7(a, &c, NULL);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, env.border_to_exclude_if_undefined( 3 ) );
		}
	protected:
		Image a, c;
};

class Sobel7x7Y: public ITest
{
	public:
		const std::string name() const override
		{
			return "Sobel7x7_Y";
		}
		Sobel7x7Y( Environment &env, RawImage &in):ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in.width(), in.height(), Format::LUMINANCE16)
		{
		}
		virtual void run(ITarget *target)override
		{
			target->sobel7x7(a, NULL, &c);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, env.border_to_exclude_if_undefined( 3 ) );
		}
	protected:
		Image a, c;
};



} /* mali_cv */
#endif /* __TEST_SOBEL_H__ */
