#include "mali_cv/Types.h"
#include "mali_cv/Context.h"

#include <numeric>

namespace mali_cv
{
class TableLookup: public ITest
{
	public:
		const std::string name() const override { return "TableLookup";}
		TableLookup( Environment &env, RawImage &_a ) : ITest(env),
			a(env.ctx, _a, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE), b(env.ctx, 256, DataType::U8)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE);
			// Initialise and set lut
			std::array<uint8_t, 256> tmp;
			std::iota( tmp.begin(), tmp.end(), 0 );
			std::for_each( tmp.begin(), tmp.end(), [](uint8_t &el){ el /= 10; } );
			b.init(tmp.data());
		}
		virtual void run(ITarget *target)override
		{
			target->tablelookup(a, b, c);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,c;
		Lut b;
};
class TableLookupS16: public ITest
{
	public:
		const std::string name() const override { return "TableLookup_S16";}

		TableLookupS16( Environment &env, RawImage &_a ) : ITest(env),
			a(env.ctx, _a, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE16), b(env.ctx, 256, DataType::S16)
		{
			ERROR_ON(_a.format() != Format::LUMINANCE16);
			// Set output to zero
			RawImage tmp_img(_a.width(), _a.height(), Format::LUMINANCE16);
			memset(tmp_img.data(), 0, tmp_img.width() * tmp_img.height() * tmp_img.pixel_size());
			c.load(tmp_img);

			// Initialise and set lut
			std::array<int16_t, 256> tmp;
			std::iota( tmp.begin(), tmp.end(), 0 );
			std::for_each( tmp.begin(), tmp.end(), [](int16_t &el){ el /= 10; } );
			b.init((void *)&tmp[0]);
		}
		virtual void run(ITarget *target)override
		{
			target->tablelookup(a, b, c);
		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output( c, env, 0 );
		}
	protected:
		Image a,c;
		Lut b;
};
}
