#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{

class TensorOpTest : public ITest
{
	const std::string &string_from_tensor_op(TensorOp op)
	{
		static std::map<TensorOp, const std::string> op_map =
		{
			{ TensorOp::ADD, "Addition" },
			{ TensorOp::SUB, "Subtraction" },
			{ TensorOp::MUL, "Multiply" },
			{ TensorOp::LUT, "Lut" },
			{ TensorOp::TRANSPOSE, "Transpose" },
		};
		return op_map[op];
	}
	public:
		const std::string name() const override
		{
			return _name;
		}
		TensorOpTest(Environment &env, TensorShape shape, DataType dt, TensorOp op)
		: ITest(env), _a(env.ctx, shape, dt), _b(env.ctx, shape, dt), _c(env.ctx, shape, dt), _op(op), _name()
		{
			// Fill input tensors randomly
			_a.random_fill(-10, 10);
			_b.random_fill(-10, 10);

			// Define operation name
			_name = "TensorOperation_" + string_from_tensor_op(op);
		}

		virtual void run(ITarget *target) override
		{
			target->tensor_ops(_a, _b, _c, _op);
		}
		virtual void validate_outputs() override
		{
			Validate::single_tensor_output( _c, env, 0 );
		}
	protected:
		Tensor _a, _b, _c;
		TensorOp _op;
		std::string _name;
};
}