#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

namespace mali_cv
{
	class ThresholdBinary: public ITest
	{
		public:
			const std::string name() const override { return "ThresholdBinary";}

			ThresholdBinary( Environment &env, RawImage &_a,uint8_t threshold, uint8_t false_value, uint8_t true_value ) : ITest(env),
				a(env.ctx, _a, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE), _threshold(threshold), _false_value(false_value), _true_value(true_value) {}

			virtual void run(ITarget *target)override
			{
				target->threshold_binary(a,c,_threshold, _false_value, _true_value);
			}
			virtual void validate_outputs() override
			{
				Validate::single_image_output( c, env, 0 );
			}
		protected:
			Image a,c;
			uint8_t _threshold;
			uint8_t _false_value;
			uint8_t _true_value;
	};

	class ThresholdRange: public ITest
	{
		public:
			const std::string name() const override { return "ThresholdRange";}

			ThresholdRange( Environment &env, RawImage &_a,uint8_t threshold,uint8_t upper, uint8_t false_value, uint8_t true_value ) : ITest(env),
				a(env.ctx, _a, CopyData::YES), c(env.ctx, _a.width(), _a.height(), Format::LUMINANCE), _threshold(threshold), _false_value(false_value), _true_value(true_value), _upper(upper) {}

			virtual void run(ITarget *target)override
			{
				target->threshold_range(a,c,_threshold, _false_value, _true_value, _upper);
			}
			virtual void validate_outputs() override
			{
				Validate::single_image_output( c, env, 0 );
			}
		protected:
			Image a,c;
			uint8_t _threshold;
			uint8_t _false_value;
			uint8_t _true_value;
			uint8_t _upper;
	};

}
