#include "mali_cv/Context.h"
#include "mali_cv/Helper.h"
#include "mali_cv/RawImage.h"
#include "mali_cv/Types.h"

#include <vector>
#include <sstream>

namespace
{
using namespace mali_cv;

template<typename T>
void compute_reference( RawImage &in, RawImage &ref)
{
	const auto mtx_in   = reinterpret_cast<const T*>(in.data());
	const auto mtx_ref = reinterpret_cast<T*>(ref.data());

	const size_t cols = in.width();
	const size_t rows = in.height();

	for (size_t y = 0; y < rows; ++y)
	{
		for(size_t x = 0; x < cols; ++x)
		{
			mtx_ref[y + x * ref.width()] = mtx_in[x + y * in.width()];
		}
	}
}
}

namespace mali_cv
{
class Transpose : public ITest
{
	public:
		const std::string name() const override
		{
			std::ostringstream s;
			s << "Transpose" << format_to_string(fmt) << "_" << a.vx.height << "x" << a.vx.width;
			return s.str();
		}
		Transpose(Environment &env, size_t rows, size_t cols, float rnd_low, float rnd_high, Format format) :
			ITest(env), a(env.ctx, cols, rows, format), b(env.ctx, rows, cols, format), ref(rows, cols, format), fmt(format)
		{
			RawImage tmp_a(cols, rows, format);

			// Initialize matrix A, B and C
			tmp_a.random_init(rnd_low, rnd_high);

			a.load(tmp_a);

			// Run internal reference implementation as in VX we do not have matrix transposition
			switch(tmp_a.element_size())
			{
				case 4:
					compute_reference<uint32_t>(tmp_a, ref);
				break;
				case 2:
					compute_reference<uint16_t>(tmp_a, ref);
				break;
				case 1:
					compute_reference<uint8_t>(tmp_a, ref);
					break;
				default:
					ERROR("Format not supported");
				break;
			}
		}
		void run(ITarget *target) override
		{
			target->transpose(a, b);
		}
		void validate_outputs() override
		{
			Validate::single_image_output(b, ref, 0.0f, env);
		}
	protected:
		Image    a;
		Image    b;
		RawImage ref;
		Format   fmt;
};
}