#include "mali_cv/Context.h"
#include "mali_cv/Types.h"

#include <cstring>

namespace mali_cv
{
class WarpAffine: public ITest
{
	public:
		const std::string name() const override
		{
			return "WarpAffine";
		}
		WarpAffine( Environment &env, RawImage &in, const float m[3][2], InterpolationMode interpolation_mode):
			ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in, CopyData::YES), mode(interpolation_mode)
		{
			memcpy( matrix, m, 6*sizeof(float) );
		}
		virtual void run(ITarget *target)override
		{
			target->warp_affine(a, c, matrix, mode);

		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output(c, env);
		}
	protected:
		Image a, c;
		float matrix[3][2];
		InterpolationMode mode;
};

class WarpPerspective: public ITest
{
	public:
		const std::string name() const override
		{
			return "WarpPerspective";
		}
		WarpPerspective( Environment &env, RawImage &in, const float m[3][3], InterpolationMode interpolation_mode):
			ITest(env), a(env.ctx, in, CopyData::YES), c(env.ctx, in, CopyData::YES), mode(interpolation_mode)
		{
			memcpy( matrix, m, 9*sizeof(float) );
		}
		virtual void run(ITarget *target)override
		{
			target->warp_perspective(a, c, matrix, mode);

		}
		virtual void validate_outputs() override
		{
			Validate::single_image_output(c, env);
		}
	protected:
		Image a, c;
		float matrix[3][3];
		InterpolationMode mode;
};
}
