#ifndef __KERNELS_H__
#define __KERNELS_H__

#define USE_ARM_COMPUTE
#define ARM_COMPUTE_NOP(...) ((void)#__VA_ARGS__)

#include <c_model.h>
#include <stdio.h>
#include <vx_internal.h>

#include "arm_compute/core/Error.h"
#include "arm_compute/core/Types.h"

#if defined(USE_ARM_COMPUTE_OPENCL)
#include "arm_compute/runtime/CL/CLFunctions.h"
#include "arm_compute/runtime/CL/CLLut.h"
#include "arm_compute/runtime/CL/CLMultiImage.h"
#include "arm_compute/runtime/CL/CLPyramid.h"
#else
#include "arm_compute/runtime/Lut.h"
#include "arm_compute/runtime/MultiImage.h"
#include "arm_compute/runtime/NEON/NEFunctions.h"
#include "arm_compute/runtime/Pyramid.h"
#endif

#include "extra_nodes.h"

#include <algorithm>
#include <memory>

#if defined(USE_ARM_COMPUTE_OPENCL)
using armcomputeMultiImage = arm_compute::CLMultiImage;
using armcomputeImage      = arm_compute::CLImage;
using armcomputeLut        = arm_compute::CLLut;
using armcomputePyramid    = arm_compute::CLPyramid;
using armcomputeTensor     = arm_compute::CLTensor;
template <class T>
using armcomputeArray                   = arm_compute::CLArray<T>;
using armcomputeDistribution1D          = arm_compute::CLDistribution1D;
using armcomputeHistogram               = arm_compute::CLHistogram;
using armcomputeHistogram               = arm_compute::CLHistogram;
using armcomputeAbsoluteDifference      = arm_compute::CLAbsoluteDifference;
using armcomputeAccumulate              = arm_compute::CLAccumulate;
using armcomputeAccumulateSquared       = arm_compute::CLAccumulateSquared;
using armcomputeAccumulateWeighted      = arm_compute::CLAccumulateWeighted;
using armcomputeArithmeticAddition      = arm_compute::CLArithmeticAddition;
using armcomputeArithmeticSubtraction   = arm_compute::CLArithmeticSubtraction;
using armcomputeBitwiseAnd              = arm_compute::CLBitwiseAnd;
using armcomputeBitwiseOr               = arm_compute::CLBitwiseOr;
using armcomputeBitwiseXor              = arm_compute::CLBitwiseXor;
using armcomputeBitwiseNot              = arm_compute::CLBitwiseNot;
using armcomputeBox3x3                  = arm_compute::CLBox3x3;
using armcomputeCannyEdge               = arm_compute::CLCannyEdge;
using armcomputeChannelCombine          = arm_compute::CLChannelCombine;
using armcomputeChannelExtract          = arm_compute::CLChannelExtract;
using armcomputeColorConvert            = arm_compute::CLColorConvert;
using armcomputeDepthConvert            = arm_compute::CLDepthConvert;
using armcomputeConvolution3x3          = arm_compute::CLConvolution3x3;
using armcomputeConvolution5x5          = arm_compute::CLConvolution5x5;
using armcomputeConvolution7x7          = arm_compute::CLConvolution7x7;
using armcomputeConvolution9x9          = arm_compute::CLConvolution9x9;
using armcomputeConvolutionRectangle    = arm_compute::CLConvolutionRectangle;
using armcomputeDilate                  = arm_compute::CLDilate;
using armcomputeEqualizeHistogram       = arm_compute::CLEqualizeHistogram;
using armcomputeErode                   = arm_compute::CLErode;
using armcomputeFastCorners             = arm_compute::CLFastCorners;
using armcomputeGaussian3x3             = arm_compute::CLGaussian3x3;
using armcomputeNonLinearFilter         = arm_compute::CLNonLinearFilter;
using armcomputeHarrisCorners           = arm_compute::CLHarrisCorners;
using armcomputeGaussianPyramidOrb      = arm_compute::CLGaussianPyramidOrb;
using armcomputeGaussianPyramidHalf     = arm_compute::CLGaussianPyramidHalf;
using armcomputeLaplacianPyramid        = arm_compute::CLLaplacianPyramid;
using armcomputeGaussianPyramidHalf     = arm_compute::CLGaussianPyramidHalf;
using armcomputeLaplacianReconstruct    = arm_compute::CLLaplacianReconstruct;
using armcomputeIntegralImage           = arm_compute::CLIntegralImage;
using armcomputeMagnitude               = arm_compute::CLMagnitude;
using armcomputeMeanStdDev              = arm_compute::CLMeanStdDev;
using armcomputeMedian3x3               = arm_compute::CLMedian3x3;
using armcomputeSobel3x3                = arm_compute::CLSobel3x3;
using armcomputeMinMaxLocation          = arm_compute::CLMinMaxLocation;
using armcomputePixelWiseMultiplication = arm_compute::CLPixelWiseMultiplication;
using armcomputeOpticalFlow             = arm_compute::CLOpticalFlow;
using armcomputePhase                   = arm_compute::CLPhase;
using armcomputeDilate                  = arm_compute::CLDilate;
using armcomputeRemap                   = arm_compute::CLRemap;
using armcomputeScale                   = arm_compute::CLScale;
using armcomputeTableLookup             = arm_compute::CLTableLookup;
using armcomputeThreshold               = arm_compute::CLThreshold;
using armcomputeWarpAffine              = arm_compute::CLWarpAffine;
using armcomputeWarpPerspective         = arm_compute::CLWarpPerspective;
#else
using armcomputeMultiImage = arm_compute::MultiImage;
using armcomputeImage      = arm_compute::Image;
using armcomputeLut        = arm_compute::Lut;
using armcomputePyramid    = arm_compute::Pyramid;
using armcomputeTensor     = arm_compute::Tensor;
template <class T>
using armcomputeArray                   = arm_compute::Array<T>;
using armcomputeDistribution1D          = arm_compute::Distribution1D;
using armcomputeHistogram               = arm_compute::NEHistogram;
using armcomputeAbsoluteDifference      = arm_compute::NEAbsoluteDifference;
using armcomputeAccumulate              = arm_compute::NEAccumulate;
using armcomputeAccumulateSquared       = arm_compute::NEAccumulateSquared;
using armcomputeAccumulateWeighted      = arm_compute::NEAccumulateWeighted;
using armcomputeArithmeticAddition      = arm_compute::NEArithmeticAddition;
using armcomputeArithmeticSubtraction   = arm_compute::NEArithmeticSubtraction;
using armcomputeBitwiseAnd              = arm_compute::NEBitwiseAnd;
using armcomputeBitwiseOr               = arm_compute::NEBitwiseOr;
using armcomputeBitwiseXor              = arm_compute::NEBitwiseXor;
using armcomputeBitwiseNot              = arm_compute::NEBitwiseNot;
using armcomputeBox3x3                  = arm_compute::NEBox3x3;
using armcomputeCannyEdge               = arm_compute::NECannyEdge;
using armcomputeChannelCombine          = arm_compute::NEChannelCombine;
using armcomputeChannelExtract          = arm_compute::NEChannelExtract;
using armcomputeColorConvert            = arm_compute::NEColorConvert;
using armcomputeDepthConvert            = arm_compute::NEDepthConvert;
using armcomputeConvolution3x3          = arm_compute::NEConvolution3x3;
using armcomputeConvolution5x5          = arm_compute::NEConvolution5x5;
using armcomputeConvolution7x7          = arm_compute::NEConvolution7x7;
using armcomputeConvolution9x9          = arm_compute::NEConvolution9x9;
using armcomputeConvolutionRectangle    = arm_compute::NEConvolutionRectangle;
using armcomputeDilate                  = arm_compute::NEDilate;
using armcomputeEqualizeHistogram       = arm_compute::NEEqualizeHistogram;
using armcomputeErode                   = arm_compute::NEErode;
using armcomputeFastCorners             = arm_compute::NEFastCorners;
using armcomputeGaussian3x3             = arm_compute::NEGaussian3x3;
using armcomputeNonLinearFilter         = arm_compute::NENonLinearFilter;
using armcomputeHarrisCorners           = arm_compute::NEHarrisCorners;
using armcomputeGaussianPyramidOrb      = arm_compute::NEGaussianPyramidOrb;
using armcomputeGaussianPyramidHalf     = arm_compute::NEGaussianPyramidHalf;
using armcomputeLaplacianPyramid        = arm_compute::NELaplacianPyramid;
using armcomputeGaussianPyramidHalf     = arm_compute::NEGaussianPyramidHalf;
using armcomputeLaplacianReconstruct    = arm_compute::NELaplacianReconstruct;
using armcomputeIntegralImage           = arm_compute::NEIntegralImage;
using armcomputeMagnitude               = arm_compute::NEMagnitude;
using armcomputeMeanStdDev              = arm_compute::NEMeanStdDev;
using armcomputeMedian3x3               = arm_compute::NEMedian3x3;
using armcomputeSobel3x3                = arm_compute::NESobel3x3;
using armcomputeMinMaxLocation          = arm_compute::NEMinMaxLocation;
using armcomputePixelWiseMultiplication = arm_compute::NEPixelWiseMultiplication;
using armcomputeOpticalFlow             = arm_compute::NEOpticalFlow;
using armcomputePhase                   = arm_compute::NEPhase;
using armcomputeDilate                  = arm_compute::NEDilate;
using armcomputeRemap                   = arm_compute::NERemap;
using armcomputeScale                   = arm_compute::NEScale;
using armcomputeTableLookup             = arm_compute::NETableLookup;
using armcomputeThreshold               = arm_compute::NEThreshold;
using armcomputeWarpAffine              = arm_compute::NEWarpAffine;
using armcomputeWarpPerspective         = arm_compute::NEWarpPerspective;
#endif

#endif /* __KERNELS_H__ */
