/*
 * Copyright (c) 2017 ARM Limited.
 *
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#ifndef __ARM_COMPUTE_TEST_VALIDATION_VX_UTILS_H__
#define __ARM_COMPUTE_TEST_VALIDATION_VX_UTILS_H__

#include "arm_compute/core/Error.h"
#include <VX/vx.h>

#define CHECK_VX(func)                       \
    if((func) != VX_SUCCESS)                 \
    {                                        \
        ARM_COMPUTE_ERROR("VX call fails!"); \
    }

#define CHECK_VX_TYPE(type, x, func) \
    type x = func;                   \
    CHECK_VX(vxGetStatus(reinterpret_cast<vx_reference>((x))));

namespace arm_compute
{
namespace test
{
namespace validation
{
inline vx_border_e convert_arm_to_vx_border_mode(BorderMode mode)
{
    switch(mode)
    {
        case BorderMode::CONSTANT:
            return VX_BORDER_CONSTANT;
        case BorderMode::REPLICATE:
            return VX_BORDER_REPLICATE;
        default:
        case BorderMode::UNDEFINED:
            return VX_BORDER_UNDEFINED;
    }
}

inline vx_border_t get_vx_border(BorderMode mode, uint8_t constant_border_value = 0)
{
    return vx_border_t{ convert_arm_to_vx_border_mode(mode), { { constant_border_value } } };
}
} // namespace validation
} // namespace test
} // namespace arm_compute
#endif /* __ARM_COMPUTE_TEST_VALIDATION_VX_UTILS_H__ */
